# !/usr/bin/python
# coding=utf-8
from uitk.slots.blender import *
from uitk.slots.subdivision import Subdivision



class Subdivision_blender(Subdivision, Slots_blender):
	def __init__(self, *args, **kwargs):
		Slots_blender.__init__(self, *args, **kwargs)
		Subdivision.__init__(self, *args, **kwargs)

		ctx = self.sb.subdivision.draggable_header.ctxMenu
		if not ctx.containsMenuItems:
			ctx.add(self.sb.ComboBox, setObjectName='cmb000', setToolTip='Subdivision Editiors.')
			ctx.add(self.sb.ComboBox, setObjectName='cmb001', setToolTip='Smooth Proxy.')
			ctx.add(self.sb.ComboBox, setObjectName='cmb002', setToolTip='Maya Subdivision Operations.')

		cmb = self.sb.subdivision.draggable_header.ctxMenu.cmb000
		items = ['Polygon Display Options']
		cmb.addItems_(items, 'Subdivision Editiors')

		cmb = self.sb.subdivision.draggable_header.ctxMenu.cmb001
		items = ['Create Subdiv Proxy','Remove Subdiv Proxy Mirror','Crease Tool','Toggle Subdiv Proxy Display', 'Both Proxy and Subdiv Display']
		cmb.addItems_(items, 'Smooth Proxy')

		cmb = self.sb.subdivision.draggable_header.ctxMenu.cmb002
		items = ['Reduce Polygons','Add Divisions','Smooth']
		cmb.addItems_(items, 'Maya Subdivision Operations')


	def cmb000(self, index=-1):
		'''Editors
		'''
		cmb = self.sb.subdivision.draggable_header.ctxMenu.cmb000

		if index>0:
			text = cmb.items[index]
			if text=='Polygon Display Options':
				pm.mel.CustomPolygonDisplayOptions() #Polygon Display Options #mel.eval("polysDisplaySetup 1;")
			cmb.setCurrentIndex(0)


	def cmb001(self, index=-1):
		'''Smooth Proxy
		'''
		cmb = self.sb.subdivision.draggable_header.ctxMenu.cmb001

		if index>0:
			text = cmb.items[index]
			if text=='Create Subdiv Proxy':
				pm.mel.SmoothProxyOptions() #'Add polygons to the selected proxy objects.' #performSmoothProxy 1;
			elif text=='Remove Subdiv Proxy Mirror':
				pm.mel.UnmirrorSmoothProxyOptions() #'Create a single low resolution mesh for a mirrored proxy setup.' #performUnmirrorSmoothProxy 1;
			elif text=='Crease Tool':
				pm.mel.polyCreaseProperties() #'Harden or soften the edges of a smooth mesh preview.' #polyCreaseValues polyCreaseContext;
			elif text=='Toggle Subdiv Proxy Display':
				pm.mel.SmoothingDisplayToggle()	#'Toggle the display of smooth shapes.' #smoothingDisplayToggle 1;
			elif text=='Both Proxy and Subdiv Display':
				pm.mel.SmoothingDisplayShowBoth() #'Display both smooth shapes' #smoothingDisplayToggle 0;
			cmb.setCurrentIndex(0)


	def cmb002(self, index=-1):
		'''Maya Subdivision Operations
		'''
		cmb = self.sb.subdivision.draggable_header.ctxMenu.cmb002

		if index>0:
			if index is cmb.items.index('Reduce Polygons'):
				pm.mel.ReducePolygonOptions()
			elif index is cmb.items.index('Add Divisions'):
				pm.mel.SubdividePolygonOptions()
			elif index is cmb.items.index('Smooth'):
				pm.mel.performPolySmooth(1)
			cmb.setCurrentIndex(0)


	def s000(self, value=None):
		'''Division Level
		'''
		value = self.sb.subdivision.s000.value()

		shapes = pm.ls(sl=1, dag=1, leaf=1)
		transforms = pm.listRelatives(shapes, p=True)
		for obj in transforms:
			if hasattr(obj, 'smoothLevel'):
				self.setAttributesMEL(obj, {'smoothLevel':value})
				pm.optionVar(intValue=['proxyDivisions', 1]) #subDiv proxy options: 'divisions'
				print(obj+': Division Level: <hl>'+str(value)+'</hl>')


	def s001(self, value=None):
		'''Tesselation Level
		'''
		value = self.sb.subdivision.s001.value()

		shapes = pm.ls(sl=1, dag=1, leaf=1)
		transforms = pm.listRelatives(shapes, p=True)
		for obj in transforms:
			if hasattr(obj, 'smoothLevel'):
				self.setAttributesMEL(obj, {'smoothTessLevel':value})
				print(obj+': Tesselation Level: <hl>'+str(value)+'</hl>')


	def b005(self):
		'''Reduce
		'''
		pm.mel.polyReduce(version=1, keepCreaseEdgeWeight=1)
		# pm.mel.ReducePolygon()


	def b008(self):
		'''Add Divisions - Subdivide Mesh
		'''
		pm.mel.SubdividePolygon()


	def b009(self):
		'''Smooth
		'''
		pm.mel.SmoothPolygon()


	def b011(self):
		'''Apply Smooth Preview
		'''
		pm.mel.performSmoothMeshPreviewToPolygon() #convert smooth mesh preview to polygons


	@staticmethod
	def smoothProxy():
		'''Subdiv Proxy
		'''
		global polySmoothBaseMesh
		polySmoothBaseMesh=[]
		#disable creating seperate layers for subdiv proxy
		pm.optionVar (intValue=["polySmoothLoInLayer",0])
		pm.optionVar (intValue=["polySmoothHiInLayer",0])
		#query smooth proxy state.
		sel = pm.mel.polyCheckSelection("polySmoothProxy", "o", 0) #mel.eval("polyCheckSelection \"polySmoothProxy\" \"o\" 0")
		
		if len(sel)==0 and len(polySmoothBaseMesh)==0:
			return 'Error: Nothing selected.'

		if len(sel)!=0:
			del polySmoothBaseMesh[:]
			for object_ in sel:
				polySmoothBaseMesh.append(object_)
		elif len(polySmoothBaseMesh) != 0:
			sel = polySmoothBaseMesh

		transform = pm.listRelatives (sel[0], fullPath=1, parent=1)
		shape = pm.listRelatives (transform[0], pa=1, shapes=1)

		#check shape for an existing output to a smoothProxy
		attachedSmoothProxies = pm.listConnections(shape[0], type="polySmoothProxy", s=0, d=1)
		if len(attachedSmoothProxies) != 0: #subdiv off
			pm.mel.smoothingDisplayToggle(0)

		#toggle performSmoothProxy
		pm.mel.performSmoothProxy(0) #toggle SubDiv Proxy;









#module name
print (__name__)
# --------------------------------------------------------------------------------------------
# Notes
# --------------------------------------------------------------------------------------------


# !/usr/bin/python
# coding=utf-8
from uitk.slots.max import *
from uitk.slots.display import Display



class Display_max(Display, Slots_max):
	def __init__(self, *args, **kwargs):
		super().__init__(*args, **kwargs)

		cmb = self.sb.display.draggable_header.ctxMenu.cmb000
		items = ['']
		cmb.addItems_(items, '')


	def cmb000(self, index=-1):
		'''Editors
		'''
		cmb = self.sb.display.draggable_header.ctxMenu.cmb000

		if index>0:
			text = cmb.items[index]
			if text=='':
				pass
			cmb.setCurrentIndex(0)


	def b000(self):
		'''Set Wireframe color
		'''
		pm.mel.objectColorPalette()


	def b001(self):
		'''Toggle Visibility
		'''
		sel = [s for s in rt.getCurrentSelection()]

		for obj in sel:
			if obj.visibility == True:
				obj.visibility = False
			else:
				obj.visibility = True


	def b002(self):
		'''Hide Selected
		'''
		sel = [s for s in rt.getCurrentSelection()]
	
		for obj in sel:
			if not obj.isHiddenInVpt:
				obj.isHidden = True


	def b003(self):
		'''Show Selected
		'''
		sel = [s for s in rt.getCurrentSelection()]
	
		for obj in sel:
			if obj.isHiddenInVpt:
				obj.isHidden = False


	def b004(self):
		'''Show Geometry
		'''
		geometry = rt.geometry

		for obj in geometry:
			if obj.isHiddenInVpt:
				obj.isHidden = False


	def b005(self):
		'''Xray Selected
		'''
		sel = [s for s in rt.getCurrentSelection()]

		for s in sel:
			s.xray = True


	def b006(self):
		'''Un-Xray All
		'''
		geometry = [g for g in rt.geometry]

		for g in geometry:
			g.xray = False


	def b007(self):
		'''Xray Other
		'''
		sel = [s for s in rt.getCurrentSelection()]
		geometry = [g for g in rt.geometry]

		for g in geometry:
			if g not in sel:
				g.xray = True


	def b008(self):
		'''Filter Objects
		'''
		mel.eval("bt_filterActionWindow;")


	def b009(self):
		'''Override Material
		'''
		if self.sb.display.chk000.isChecked(): #override with UV checker material
			self.toggleMaterialOverride(checker=1)
		else:
			self.toggleMaterialOverride()


	def b010(self):
		''''''
		pass


	def b011(self):
		'''Toggle Component ID Display
		'''
		index = ptk.cycle([0,1,2,3,4], 'componentID')

		visible = pm.polyOptions (query=1, displayItemNumbers=1)
		dinArray = [[1,0,0,0],[0,1,0,0],[0,0,1,0],[0,0,0,1]]

		if index == 4:
			i=0
			for _ in range(4):
				if visible[i] == True:
					pm.polyOptions (relative=1, displayItemNumbers=dinArray[i], activeObjects=1)
				i+=1

		if visible[index] != True and index != 4:
			pm.polyOptions (relative=1, displayItemNumbers=dinArray[index], activeObjects=1)

			i=0
			for _ in range(4):
				if visible[i] == True and i != index:
					pm.polyOptions (relative=1, displayItemNumbers=dinArray[i], activeObjects=1)
				i+=1

		if index == 0:
			self.sb.messageBox('<hl>Vertex IDs</hl>.') #[1,0,0,0]
			return
		elif index == 1:
			self.sb.messageBox('<hl>Edge IDs</hl>.') #[0,1,0,0]
			return
		elif index == 2:
			self.sb.messageBox('<hl>Face IDs</hl>.') #[0,0,1,0]
			return
		elif index == 3:
			self.sb.messageBox('<hl>Component IDs (UV)</hl>.') #[0,0,0,1]
			return
		elif index == 4:
			self.sb.messageBox('Component ID <hl>Off</hl>.')
			return


	def b012(self):
		'''Wireframe Non Active (Wireframe All But The Selected Item)
		'''
		viewport = rt.viewport.activeViewport

		state = ptk.cycle([0,1], 'wireframeInactive')

		if state:
			if not rt.viewport.isWire():
				self.maxUiSetChecked("415", 62, 163, True) #set viewport to wireframe
			self.maxUiSetChecked("40212", 62, 130, True) #Shade selected objects Checked
		else:
			self.maxUiSetChecked("40212", 62, 130, False) #Shade selected objects unchecked


	def b013(self):
		'''Viewport Configuration
		'''
		maxEval('actionMan.executeAction 0 "40023"')


	def b021(self):
		'''Template Selected
		'''
		sel = [s for s in rt.getCurrentSelection()]

		for obj in sel:
			if obj.isFrozen == True:
				obj.isFrozen = False
			else:
				obj.isFrozen = True


	def b024(self):
		'''Polygon Display Options
		'''
		mel.eval("CustomPolygonDisplayOptions")
		# mel.eval("polysDisplaySetup 1;")








#module name
print (__name__)
# --------------------------------------------------------------------------------------------
# Notes
# --------------------------------------------------------------------------------------------





# !/usr/bin/python
# coding=utf-8
from uitk.slots.maya import *
from uitk.slots.scene import Scene


class Scene_maya(Scene, Slots_maya):
	def __init__(self, *args, **kwargs):
		super().__init__(*args, **kwargs)

		cmb = self.sb.scene.draggable_header.ctxMenu.cmb000
		items = ['Node Editor', 'Outlinder', 'Content Browser', 'Optimize Scene Size', 'Prefix Hierarchy Names', 'Search and Replace Names']
		cmb.addItems_(items, 'Maya Scene Editors')


	def cmb000(self, index=-1):
		'''Editors
		'''
		cmb = self.sb.scene.draggable_header.ctxMenu.cmb000

		if index>0:
			text = cmb.items[index]
			if text=='Node Editor':
				pm.mel.eval('NodeEditorWindow;') #
			elif text=='Outlinder':
				pm.mel.eval('OutlinerWindow;') #
			elif text=='Content Browser':
				pm.mel.eval('ContentBrowserWindow;') #
			elif text=='Optimize Scene Size':
				pm.mel.eval('cleanUpScene 2;')
			elif text=='Prefix Hierarchy Names':
				pm.mel.eval('prefixHierarchy;') #Add a prefix to all hierarchy names.
			elif text=='Search and Replace Names':
				pm.mel.eval('SearchAndReplaceNames;') #performSearchReplaceNames 1; #Rename objects in the scene.
			cmb.setCurrentIndex(0)


	def tb000(self, state=None):
		'''Convert Case
		'''
		tb = self.sb.scene.tb000

		case = tb.ctxMenu.cmb001.currentText()

		selection = pm.ls(sl=1)
		objects = selection if selection else pm.ls(objectsOnly=1)
		mtk.Edit.setCase(objects, case)


	def tb001(self, state=None):
		'''Convert Case
		'''
		tb = self.sb.scene.tb001

		alphanumeric = tb.ctxMenu.chk005.isChecked()
		stripTrailingInts = tb.ctxMenu.chk002.isChecked()
		stripTrailingAlpha = tb.ctxMenu.chk003.isChecked()
		reverse = tb.ctxMenu.chk004.isChecked()

		selection = pm.ls(sl=1, objectsOnly=1, type='transform')
		mtk.Edit.setSuffixByObjLocation(selection, alphanumeric=alphanumeric, stripTrailingInts=stripTrailingInts, stripTrailingAlpha=stripTrailingAlpha, reverse=reverse)


	def b000(self):
		'''Rename
		'''
		find = self.sb.scene.t000.text() #an asterisk denotes startswith*, *endswith, *contains* 
		to = self.sb.scene.t001.text()
		regEx = self.sb.scene.t000.ctxMenu.chk001.isChecked()
		ignoreCase = self.sb.scene.t000.ctxMenu.chk000.isChecked()

		selection = pm.ls(sl=1)
		objects = selection if selection else pm.ls(objectsOnly=1)
		mtk.Edit.rename(objects, to, find, regEx=regEx, ignoreCase=ignoreCase)

# --------------------------------------------------------------------------------------------









#module name
print (__name__)
# --------------------------------------------------------------------------------------------
# Notes
# --------------------------------------------------------------------------------------------


# !/usr/bin/python
# coding=utf-8
from uitk.slots.blender import *
from uitk.slots.nurbs import Nurbs



class Nurbs_blender(Nurbs, Slots_blender):
	def __init__(self, *args, **kwargs):
		Slots_blender.__init__(self, *args, **kwargs)
		Nurbs.__init__(self, *args, **kwargs)

		cmb = self.sb.nurbs.draggable_header.ctxMenu.cmb000
		items = ['Project Curve','Duplicate Curve','Create Curve from Poly','Bend Curve', 'Curl Curve','Modify Curve Curvature','Smooth Curve','Straighten Curves','Extrude Curves','Revolve Curves','Loft Curves','Planar Curves','Insert Isoparms','Insert Knot','Rebuild Curve','Extend Curve', 'Extend Curve On Surface']
		cmb.addItems_(items, 'Curve Editors')

		cmb = self.sb.nurbs.cmb001
		items = ['Ep Curve Tool','CV Curve Tool','Bezier Curve Tool','Pencil Curve Tool','2 Point Circular Arc','3 Point Circular Arc']
		cmb.addItems_(items, 'Create Curve')


	def cmb000(self, index=-1):
		'''Maya Curve Operations
		'''
		cmb = self.sb.nurbs.draggable_header.ctxMenu.cmb000

		if index>0:
			text = cmb.items[index]
			if text=='Project Curve':
				mel.eval("ProjectCurveOnSurfaceOptions;")
			elif text=='Duplicate Curve':
				mel.eval("DuplicateCurveOptions;")
			elif text=='Create Curve from Poly':
				mel.eval("CreateCurveFromPolyOptions")
			elif text=='Bend Curve':
				mel.eval("BendCurvesOptions;")
			elif text=='Curl Curve':
				mel.eval("CurlCurvesOptions;")
			elif text=='Modify Curve Curvature':
				mel.eval("ScaleCurvatureOptions;")
			elif text=='Smooth Curve':
				mel.eval("SmoothHairCurvesOptions;")
			elif text=='Straighten Curves':
				mel.eval("StraightenCurvesOptions;")
			elif text=='Extrude Curves':
				mel.eval("ExtrudeOptions;")
			elif text=='Revolve Curves':
				mel.eval("RevolveOptions;")
			elif text=='Loft Curves':
				mel.eval("LoftOptions;")
			elif text=='Planar Curves':
				mel.eval("PlanarOptions;")
			elif text=='Insert Isoparms':
				mel.eval("InsertIsoparmsOptions;")
			elif text=='Insert Knot':
				mel.eval("InsertKnotOptions;")
			elif text=='Rebuild Curve':
				mel.eval("RebuildCurveOptions;")
			elif text=='Extend Curve':
				mel.eval("ExtendCurveOptions;")
			elif text=='Extend Curve On Surface':
				mel.eval("ExtendCurveOnSurfaceOptions;")
			cmb.setCurrentIndex(0)


	def cmb001(self, index=-1):
		'''Create: Curve
		'''
		cmb = self.sb.nurbs.cmb001

		if index>0:
			text = cmb.items[index]
			if text=='Ep Curve Tool':
				mel.eval('EPCurveToolOptions;') #mel.eval('EPCurveTool;')
			elif text=='CV Curve Tool':
				mel.eval('CVCurveToolOptions') #mel.eval('CVCurveTool')
			elif text=='Bezier Curve Tool':
				mel.eval('CreateBezierCurveToolOptions') #mel.eval('CreateBezierCurveTool;')
			elif text=='Pencil Curve Tool':
				mel.eval('PencilCurveToolOptions;') #mel.eval('PencilCurveTool;')
			elif text=='2 Point Circular Arc':
				mel.eval('TwoPointArcToolOptions;') #mel.eval("TwoPointArcTool;")
			elif text=='3 Point Circular Arc':
				mel.eval("ThreePointArcToolOptions;") #mel.eval("ThreePointArcTool;")
			cmb.setCurrentIndex(0)


	@Slots_blender.attr
	def tb000(self, state=None):
		'''Revolve
		'''
		tb = self.sb.nurbs.tb000

		degree = tb.ctxMenu.s002.value()
		startSweep = tb.ctxMenu.s003.value()
		endSweep = tb.ctxMenu.s004.value()
		sections = tb.ctxMenu.s005.value()
		range_ = tb.ctxMenu.chk006.isChecked()
		polygon = 1 if tb.ctxMenu.chk007.isChecked() else 0
		autoCorrectNormal = tb.ctxMenu.chk008.isChecked()
		useTolerance = tb.ctxMenu.chk009.isChecked()
		tolerance = tb.ctxMenu.s006.value()

		return pm.revolve(curves, po=polygon, rn=range_, ssw=startSweep, esw=endSweep, ut=useTolerance, tol=tolerance, degree=degree, s=sections, ulp=1, ax=[0,1,0])


	# @Slots_blender.attr
	def tb001(self, state=None):
		'''Loft
		'''
		tb = self.sb.nurbs.tb001

		uniform = tb.ctxMenu.chk000.isChecked()
		close = tb.ctxMenu.chk001.isChecked()
		degree = tb.ctxMenu.s000.value()
		autoReverse = tb.ctxMenu.chk002.isChecked()
		sectionSpans = tb.ctxMenu.s001.value()
		range_ = tb.ctxMenu.chk003.isChecked()
		polygon = 1 if tb.ctxMenu.chk004.isChecked() else 0
		reverseSurfaceNormals = tb.ctxMenu.chk005.isChecked()
		angleLoftBetweenTwoCurves = tb.ctxMenu.chk010.isChecked()
		angleLoftSpans = tb.ctxMenu.s007.value()

		self.loft(uniform=uniform, close=close, degree=degree, autoReverse=autoReverse, sectionSpans=sectionSpans, range_=range_, polygon=polygon, reverseSurfaceNormals=reverseSurfaceNormals, angleLoftBetweenTwoCurves=angleLoftBetweenTwoCurves, angleLoftSpans=angleLoftSpans)


	def b012(self):
		'''Project Curve
		'''
		mel.eval("projectCurve;") #ProjectCurveOnMesh;


	def b014(self):
		'''Duplicate Curve
		'''
		pm.mel.DuplicateCurve()


	def b016(self):
		'''Extract Curve
		'''
		try:
			pm.mel.CreateCurveFromPoly()

		except Exception as error:
			objects = pm.ls(sl=1, objectsOnly=1)
			sel_edges = mtk.Cmpt.getComponents('edges', flatten=1)
			edge_rings = self.getContigiousEdges(sel_edges)
			multi = len(edge_rings)>1

			for edge_ring in edge_rings:
				pm.select(edge_ring)
				if multi:
					pm.polyToCurve(form=2, degree=3, conformToSmoothMeshPreview=True) #degree: 1=linear,2= ,3=cubic,5= ,7=
				else:
					return pm.polyToCurve(form=2, degree=3, conformToSmoothMeshPreview=True) #degree: 1=linear,2= ,3=cubic,5= ,7=


	def b018(self):
		'''Lock Curve
		'''
		pm.mel.LockCurveLength()


	def b019(self):
		'''Unlock Curve
		'''
		pm.mel.UnlockCurveLength()


	def b020(self):
		'''Bend Curve
		'''
		pm.mel.BendCurves()


	def b022(self):
		'''Curl Curve
		'''
		pm.mel.CurlCurves()


	def b024(self):
		'''Modify Curve Curvature
		'''
		pm.mel.ScaleCurvature()


	def b026(self):
		'''Smooth Curve
		'''
		pm.mel.SmoothHairCurves()


	def b028(self):
		'''Straighten Curve

		'''
		pm.mel.StraightenCurves()


	def b030(self):
		'''Extrude

		'''
		pm.mel.Extrude()


	def b036(self):
		'''Planar
		'''
		pm.mel.Planar()


	def b038(self):
		'''Insert Isoparm
		'''
		pm.mel.InsertIsoparms()


	def b040(self):
		'''Edit Curve Tool
		'''
		pm.mel.CurveEditTool()


	def b041(self):
		'''Attach Curve
		'''
		pm.mel.AttachCurveOptions()


	def b042(self):
		'''Detach Curve
		'''
		pm.mel.DetachCurve()


	def b043(self):
		'''Extend Curve
		'''
		pm.mel.ExtendCurveOptions()


	def b045(self):
		'''Cut Curve
		'''
		pm.mel.CutCurve()


	def b046(self):
		'''Open/Close Curve
		'''
		pm.mel.OpenCloseCurve()


	def b047(self):
		'''Insert Knot
		'''
		pm.mel.InsertKnot()


	def b049(self):
		'''Add Points Tool
		'''
		pm.mel.AddPointsTool()


	def b051(self):
		'''Reverse Curve

		'''
		mel.eval("reverse;")


	def b052(self):
		'''Extend Curve
		'''
		pm.mel.ExtendCurve()


	def b054(self):
		'''Extend On Surface
		'''
		pm.mel.ExtendCurveOnSurface()


	@Slots_blender.undoChunk
	def loft(self, uniform=True, close=False, degree=3, autoReverse=False, sectionSpans=1, range_=False, polygon=True, reverseSurfaceNormals=True, angleLoftBetweenTwoCurves=False, angleLoftSpans=6):
		'''Create a loft between two selections.

		Parameters:
			uniform (bool): The resulting surface will have uniform parameterization in the loft direction. If set to false, the parameterization will be chord length.
			close (bool): The resulting surface will be closed (periodic) with the start (end) at the first curve. If set to false, the surface will remain open.
			degree (int): The degree of the resulting surface.
			autoReverse (bool): The direction of the curves for the loft is computed automatically. If set to false, the values of the multi-use reverse flag are used instead.
			sectionSpans (int): The number of surface spans between consecutive curves in the loft.
			range_ (bool): Force a curve range on complete input curve.
			polygon (bool): The object created by this operation.
			reverseSurfaceNormals (bool): The surface normals on the output NURBS surface will be reversed. This is accomplished by swapping the U and V parametric directions.
			angleLoftBetweenTwoCurves (bool): Perform a loft at an angle between two selected curves or polygon edges (that will be extracted as curves).
			angleLoftSpans (int): Angle loft: Number of duplicated points (spans).

		Return:
			(obj) nurbsToPoly history node.
		'''
		# pm.undoInfo(openChunk=1)
		sel = pm.ls(sl=1)

		if len(sel)>1:
			if angleLoftBetweenTwoCurves:
				start, end = sel[:2] #get the first two selected edge loops or curves.
				result = self.angleLoftBetweenTwoCurves(start, end, count=angleLoftSpans, cleanup=True, uniform=uniform, close=close, autoReverse=autoReverse, degree=degree, sectionSpans=sectionSpans, range=range_, polygon=0, reverseSurfaceNormals=reverseSurfaceNormals)
			else:
				result = pm.loft(sel, u=uniform, c=close, ar=autoReverse, d=degree, ss=sectionSpans, rn=range_, po=0, rsn=reverseSurfaceNormals)
		else:
			return '# Error: Operation requires the selection of two curves or polygon edge sets. #'

		if polygon: #convert nurb surface to polygon.
			converted = pm.nurbsToPoly(result[0], mnd=1,  f=3, pt=1, pc=200, chr=0.1, ft=0.01, mel=0.001, d=0.1, ut=1, un=3, vt=1, vn=3, uch=0, ucr=0, cht=0.2, es=0, ntr=0, mrt=0, uss=1)
			for obj in result:
				try:
					pm.delete(obj)
				except:
					pass
			result=converted

		# pm.undoInfo(closeChunk=1)
		return result


	@Slots_blender.undoChunk
	def createCurveBetweenTwoObjects(self, start, end):
		'''Create a bezier curve between starting and end object(s).

		Parameters:
			start () = Starting object(s).
			end () = Ending object(s).

		Return:
			(obj) Bezier curve. 
		'''
		# pm.undoInfo(openChunk=1)
		p1 = pm.objectCenter(start)
		p2 = pm.objectCenter(end)
		hypotenuse = Slots.getDistBetweenTwoPoints(p1, p2)

		v1, v2 = self.getCrossProductOfCurves([start, end], normalize=1, values=1)
		v3a = Slots.getVectorFromTwoPoints(p1, p2)
		v3b = Slots.getVectorFromTwoPoints(p2, p1)

		a1 = Slots.getAngleFrom2Vectors(v1, v3a, degree=1) #Slots_blender.getAngleFrom3Points(v1, p1, p2, degree=1)
		a2 = Slots.getAngleFrom2Vectors(v2, v3b, degree=1) #Slots_blender.getAngleFrom3Points(v2, p1, p2, degree=1)
		a3 = Slots.getAngleFrom2Vectors(v1, v2, degree=1)

		d1, d2 = Slots.getTwoSidesOfASATriangle(a2, a1, hypotenuse) #get length of sides 1 and 2.

		p_from_v1 = Slots.movePointAlongVectorRelativeToPoint(p1, p2, v1, d1)
		p_from_v2 = Slots.movePointAlongVectorRelativeToPoint(p2, p1, v2, d2)
		p3 = Slots.getCenterPointBetweenTwoPoints(p_from_v1, p_from_v2)

		if d1<d2:
			min_dist = d1
			max_vect = Slots.getVectorFromTwoPoints(p2, p3)
		else:
			min_dist = d2
			max_vect = Slots.getVectorFromTwoPoints(p1, p3)
			p1, p2 = p2, p1

		# pm.spaceLocator(position=p1); pm.spaceLocator(position=p2); pm.spaceLocator(position=p3)

		p4 = Slots.movePointRelative(p3, min_dist, max_vect); #pm.spaceLocator(position=p4)
		p5 = Slots.getCenterPointBetweenTwoPoints(p4, p1); #pm.spaceLocator(position=p5)
		p6 = Slots.getCenterPointBetweenTwoPoints(p3, p5); #pm.spaceLocator(position=p6)

		#add weighting to the curve points.
		p1w, p3w, p4w, p2w = [
			(p1[0], p1[1], p1[2], 1),
			(p3[0], p3[1], p3[2], 4),
			(p4[0], p4[1], p4[2], 10),
			(p2[0], p2[1], p2[2], 1),
		]

		result = pm.curve(pw=[p1w, p3w, p4w, p2w], k=[0,0,0,1,1,1], bezier=1)
		# pm.undoInfo(closeChunk=1)

		return result


	@Slots_blender.undoChunk
	def duplicateAlongCurve(self, path, start, count=6, geometry='Instancer'):
		'''Duplicate objects along a given curve using MASH.

		Parameters:
			path (obj): The curve to use as a path.
			start () = Starting object.
			count (int): The number of duplicated objects. (point count on the MASH network)
			geometry (str): Particle instancer or mesh instancer (Repro node). (valid: 'Mesh' (default), 'Instancer')

		Return:
			(list) The duplicated objects in order of start to end.
		'''
		# pm.undoInfo(openChunk=1)
		#create a MASH network
		import MASH.api as mapi
		import utils_maya.mash_utils_maya

		mashNW = mapi.Network()
		mashNW.MTcreateNetwork(start, geometry=geometry, hideOnCreate=False) #mash_utils_maya module (derived from 'createNetwork')

		curveNode = pm.ls(mashNW.addNode('MASH_Curve').name)[0]
		pm.connectAttr(path.worldSpace[0], curveNode.inCurves[0], force=1)

		pm.setAttr(curveNode.stopAtEnd, 1) #0=off, 1=on
		pm.setAttr(curveNode.clipStart, 0)
		pm.setAttr(curveNode.clipEnd, 1)
		pm.setAttr(curveNode.equalSpacing, 1)
		pm.setAttr(curveNode.timeStep, 1)
		pm.setAttr(curveNode.curveLengthAffectsSpeed, 1)

		distNode = pm.ls(mashNW.distribute)[0]
		pm.setAttr(distNode.pointCount, count)
		pm.setAttr(distNode.amplitudeX, 0)

		instNode = pm.ls(mashNW.instancer)[0]
		baked_curves = mashNW.MTbakeInstancer(instNode) #mash_utils_maya module (derived from 'MASHbakeInstancer')

		result=[start]
		for curve in reversed(baked_curves):
			result.append(curve)

		pm.delete(mashNW.waiter.name()) #delete the MASH network.
		# pm.undoInfo(closeChunk=1)

		return result


	@Slots_blender.undoChunk
	def angleLoftBetweenTwoCurves(self, start, end, count=6, cleanup=False, uniform=1, close=0, autoReverse=0, 
										degree=3, sectionSpans=1, range=0, polygon=1, reverseSurfaceNormals=0):
		'''Perform a loft between two nurbs curves or polygon sets of edges (that will be extracted as curves).

		Parameters:
			start (list): Starting edges.
			end (list): Ending edges.
			count (int): Section count.
			cleanup (bool): Delete the start, end, and any additional construction curves upon completion.

		Return:
			(list) Loft object name and node name.
		'''
		# pm.undoInfo(openChunk=1)
		if pm.objectType(start)=='mesh': #vs. 'nurbsCurve'
			start, startNode = pm.polyToCurve(start, form=2, degree=3, conformToSmoothMeshPreview=1) #extract curve from mesh
		mtk.XformresetTranslation(start) #reset the transforms to world origin.

		if pm.objectType(end)=='mesh': #vs. 'nurbsCurve'
			end, endNode = pm.polyToCurve(end, form=2, degree=3, conformToSmoothMeshPreview=1) #extract curve from mesh
		mtk.XformresetTranslation(end) #reset the transforms to world origin.

		path = self.createCurveBetweenTwoObjects(start, end)
		curves = self.duplicateAlongCurve(path, start, count=count)

		#align end
		# find curve start using closestPointOnCurve method, 
		# and rebuild the end curve to match the duplicated curves.
		# then reverse.
		# pm.reverseCurve(end, rpo=1)

		result = pm.loft(curves, u=uniform, c=close, ar=autoReverse, d=degree, ss=sectionSpans, rn=range, po=polygon, rsn=reverseSurfaceNormals)

		if cleanup: #perform cleanup by deleting construction curves.
			try:
				curves_parent = pm.listRelatives(curves[1], parent=1)
				pm.delete(curves_parent)
				pm.delete(end)
				pm.delete(path)
				pm.delete(start)
			except Exception as e:
				print(e)
		# pm.undoInfo(closeChunk=1)

		return result


	@Slots_blender.undoChunk
	def getClosestCV(self, x, curves, tolerance=0.0):
		'''Find the closest control vertex between the given vertices, CVs, or objects and each of the given curves.

		Parameters:
			x (str)(obj)(list): Polygon vertices, control vertices, objects, or points given as (x,y,z) tuples.
			curves (str)(obj)(list): The reference object in which to find the closest CV for each vertex in the list of given vertices.
			tolerance (int)(float) = Maximum search distance. Default is 0.0, which turns off the tolerance flag.

		Return:
			(dict) closest vertex/cv pairs (one pair for each given curve) ex. {<vertex from set1>:<vertex from set2>}.

		ex. vertices = getComponents(objects, 'vertices')
			closestVerts = getClosestCV(curve0, curves)
		'''
		# pm.undoInfo(openChunk=True)
		x = pm.ls(x, flatten=1) #assure x arg is a list (if given as str or single object).

		npcNode = pm.ls(pm.createNode('nearestPointOnCurve'))[0] #create a nearestPointOnCurve node.

		result={}
		for curve in pm.ls(curves):

			pm.connectAttr(curve.worldSpace, npcNode.inputCurve, force=1) #Connect the curve's worldSpace geometry to the npc node.

			for i in x:
				if not isinstance(i, (tuple, list, set)):
					pos = pm.pointPosition(i)
				else:
					pos = i
				pm.setAttr(npcNode.inPosition, pos)

				distance = Slots.getDistBetweenTwoPoints(pos, pm.getAttr(npcNode.position))
				p = pm.getAttr(npcNode.parameter)
				if not tolerance:
					result[i] = p
				elif distance < tolerance:
					result[i] = p

		pm.delete(npcNode)
		# pm.undoInfo(closeChunk=True)

		return result


	def getCvInfo(self, c, returnType='cv', filter_=[]):
		'''Get a dict containing CV's of the given curve(s) and their corresponding point positions (based on Maya's pointOnCurve command).

		Parameters:
			- c (str)(obj)(list): Curves or CVs to get CV info from.
			- returnType (str): The desired returned values. Default is 'cv'.
				valid values are: 
					'cv' = Return a list of all CV's for the given curves.
					'count' = Return an integer representing the total number of cvs for each of the curves given.
					'parameter', 'position', 'index', 'localPosition', 'tangent', 'normalizedTangent', 'normal', 'normalizedNormal', 'curvatureRadius', 'curvatureCenter'
					= Return a dict with CV's as keys and the returnType as their corresponding values.
				ex. {NurbsCurveCV(u'polyToCurveShape7.cv[5]'): [-12.186520865542082, 15.260936896515751, -369.6159740743584]}
			- filter_ (str)(obj)(list): Value(s) to filter for in the returned results.

		Return:
			(dict)(list)(int) dependant on returnType.

		ex. cv_tan = getCvInfo(curve.cv[0:2],'tangent') #get CV tangents for cvs 0-2.
		ex. cvParam = getCvInfo(curve, 'parameters') #get the curves CVs and their corresponding U parameter values.
		ex. filtered = getCvInfo(<curve>, 'normal', <normal>) #filter results for those that match the given value.
		'''
		result={}
		for curve in pm.ls(c):

			if '.cv' in str(curve): #if CV given.
				cvs = curve
				curve = pm.listRelatives(cvs, parent=1)
			else: #if curve(s) given
				cvs = curve.cv

			parameters = self.getClosestCV(cvs, curve) #use getClosestCV to get the parameter location for each of the curves CVs.
			for cv, p in parameters.items():

				if returnType=='position': # Get cv position
					v = pm.pointOnCurve(curve, parameter=p, position=True)
				elif returnType=='localPosition':
					v = pm.getAttr(cv) # local cv position
				elif returnType=='tangent': # Get cv tangent
					v = pm.pointOnCurve(curve, parameter=p, tangent=True)
				elif returnType=='normalizedTangent':
					v = pm.pointOnCurve(curve, parameter=p, normalizedTangent=True)
				elif returnType=='normal': # Get cv normal
					v = pm.pointOnCurve(curve, parameter=p, normal=True)
				elif returnType=='normalizedNormal':
					v = pm.pointOnCurve(curve, parameter=p, normalizedNormal=True) #Returns the (x,y,z) normalized normal of curve1 at parameter 0.5.
				elif returnType=='curvatureRadius': # Get cv curvature
					v = pm.pointOnCurve(curve, parameter=p, curvatureRadius=True) #Returns the curvature radius of curve1 at parameter 0.5.
				elif returnType=='curvatureCenter':
					v = pm.pointOnCurve(curve, parameter=p, curvatureCenter=True)
				elif returnType=='parameter': # Return the CVs parameter.
					v = p
				elif returnType=='count': # total number of cv's for the curve.
					result[curve] = len(self.getCvInfo(curve))
					break
				elif returnType=='index': # index of the cv
					s = str(cv)
					v = int(s[s.index('[')+1:s.index(']')])
				else:
					v = None

				result[cv] = v

		if returnType=='cv':
			result = result.keys()

		if filter_:
			if not isinstance(filter_, (tuple, set, list)):
				filter_ = list(filter_)
			try:
				result = {k:v for k,v in result.items() if any((v in filter_, v==filter_))}
			except AttributeError:
				result = [i for i in result if any((i in filter_, i==filter_))]

		if len(result)==1:
			try:
				result = list(result.values())[0]
			except (AttributeError, TypeError):
				result = result[0]

		return result


	def getCrossProductOfCurves(self, curves, normalize=1, values=False):
		'''Get the cross product of two vectors using points derived from the given curves.

		Parameters:
			curves (str)(obj)(list): Nurbs curve(s).
			normalize (float) = (0) Do not normalize. (1) Normalize standard. (value other than 0 or 1) Normalize using the given float value as desired length.
			values (bool): Return only a list of the cross product vector values [(<Vx>, <Vy>, <Vz>)] instead of the full dict {<curve1>:(<Vx>, <Vy>, <Vz>)}.

		Return:
			(dict)(list)
		'''
		result={}
		for curve in pm.ls(curves):
			p0 = pm.objectCenter(curve)

			cvs = mtk.Cmpt.getComponents(curve, 'cv', returnType='object', flatten=1)
			cvPos = self.getCvInfo(curve, 'position')
			p1 = cvPos[cvs[0]]
			p2 = cvPos[cvs[(len(cvs)/2)]]

			n1 = Slots.getCrossProduct(p0, p1, p2, normalize=normalize)

			result[curve] = n1

		if values:
			result = list(result.values())
		return result









#module name
print (__name__)
# --------------------------------------------------------------------------------------------
# Notes
# --------------------------------------------------------------------------------------------


# !/usr/bin/python
# coding=utf-8
from uitk.slots.maya import *
from uitk.slots.editors import Editors


class Editors_maya(Editors, Slots_maya):
	def __init__(self, *args, **kwargs):
		super().__init__(*args, **kwargs)

		# tree = self.sb.editors_lower_submenu.tree000
		# tree.expandOnHover = True

		# l = ['General Editors', 'Modeling Editors', 'Animation Editors', 'Rendering Editors', 'Relationship Editors']
		# [tree.add('QLabel', childHeader=s, setText=s) for s in l] #root

		# l = ['Attribute Editor', 'Channel Box', 'Layer Editor', 'Content Browser', 'Tool Settings', 'Hypergraph: Hierarchy', 'Hypergraph: Connections', 'Viewport', 'Adobe After Effects Live Link', 'Asset Editor', 'Attribute Spread Sheet', 'Component Editor', 'Channel Control', 'Display Layer Editor', 'File Path Editor', 'Namespace Editor', 'Script Editor', 'Command Shell', 'Profiler', 'Evaluation Toolkit']
		# [tree.add('QLabel', 'General Editors', setText=s) for s in l]

		# l = ['Modeling Toolkit', 'Paint Effects', 'UV Editor', 'XGen Editor', 'Crease Sets']
		# [tree.add('QLabel', 'Modeling Editors', setText=s) for s in l]

		# l = ['Graph Editor', 'Time Editor', 'Trax Editor', 'Camera Sequencer', 'Dope Sheet', 'Quick Rig', 'HumanIK', 'Shape Editor', 'Pose Editor', 'Expression Editor']
		# [tree.add('QLabel', 'Animation Editors', setText=s) for s in l]

		# l = ['Render View', 'Render Settings', 'Hypershade', 'Render Setup', 'Light Editor', 'Custom Stereo Rig Editor', 'Rendering Flags', 'Shading Group Attributes']
		# [tree.add('QLabel', 'Rendering Editors', setText=s) for s in l]

		# l = ['Animation Layers', 'Camera Sets', 'Character Sets', 'Deformer Sets', 'Display Layers', 'Dynamic Relationships', 'Light Linking: Light Centric','Light Linking: Object Centric', 'Partitions', 'Render Pass Sets', 'Sets', 'UV Linking: Texture-Centric', 'UV Linking: UV-Centric', 'UV Linking: Paint Effects/UV', 'UV Linking: Hair/UV']
		# [tree.add('QLabel', 'Relationship Editors', setText=s) for s in l]


	def tree000(self, wItem=None, column=None):
		'''
		'''
		tree = self.sb.editors_lower_submenu.tree000

		if not any([wItem, column]): # code here will run before each show event. generally used to refresh tree contents. -----------------------------
			return

		widget = tree.getWidget(wItem, column)
		text = tree.getWidgetText(wItem, column)
		header = tree.getHeaderFromColumn(column)

		self.sb.parent().hide() #hide the menu before opening an external editor.

		if header=='General Editors':
			if text=='Attribute Editor':
				pm.mel.AttributeEditor()
			if text=='Channel Box':
				pm.mel.OpenChannelBox()
			if text=='Layer Editor':
				pm.mel.OpenLayerEditor()
			if text=='Content Browser':
				pm.mel.OpenContentBrowser()
			if text=='Tool Settings':
				pm.mel.ToolSettingsWindow()
			if text=='Hypergraph: Hierarchy':
				pm.mel.HypergraphHierarchyWindow()
			if text=='Hypergraph: Connections':
				pm.mel.HypergraphDGWindow()
			if text=='Viewport':
				pm.mel.DisplayViewport()
			if text=='Adobe After Effects Live Link':
				pm.mel.OpenAELiveLink()
			if text=='Asset Editor':
				pm.mel.AssetEditor()
			if text=='Attribute Spread Sheet':
				pm.mel.SpreadSheetEditor()
			if text=='Component Editor':
				pm.mel.ComponentEditor()
			if text=='Channel Control':
				pm.mel.ChannelControlEditor()
			if text=='Display Layer Editor':
				pm.mel.DisplayLayerEditorWindow()
			if text=='File Path Editor':
				pm.mel.FilePathEditor()
			if text=='Namespace Editor':
				pm.mel.NamespaceEditor()
			if text=='Script Editor':
				pm.mel.ScriptEditor()
			if text=='Command Shell':
				pm.mel.CommandShell()
			if text=='Profiler':
				pm.mel.ProfilerTool()
			if text=='Evaluation Toolkit':
				pm.mel.EvaluationToolkit()

		elif header=='Modeling Editors':
			if text=='Modeling Toolkit':
				pm.mel.OpenModelingToolkit()
			if text=='Paint Effects':
				pm.mel.PaintEffectsWindow()
			if text=='UV Editor':
				pm.mel.TextureViewWindow()
			if text=='XGen Editor':
				pm.mel.OpenXGenEditor()
			if text=='Crease Sets':
				pm.mel.OpenCreaseEditor()

		elif header=='Animation Editors':
			if text=='Graph Editor':
				pm.mel.GraphEditor()
			if text=='Time Editor':
				pm.mel.TimeEditorWindow()
			if text=='Trax Editor':
				pm.mel.CharacterAnimationEditor()
			if text=='Camera Sequencer':
				pm.mel.SequenceEditor()
			if text=='Dope Sheet':
				pm.mel.DopeSheetEditor()
			if text=='Quick Rig':
				pm.mel.QuickRigEditor()
			if text=='HumanIK':
				pm.mel.HIKCharacterControlsTool()
			if text=='Shape Editor':
				pm.mel.ShapeEditor()
			if text=='Pose Editor':
				pm.mel.PoseEditor()
			if text=='Expression Editor':
				pm.mel.ExpressionEditor()

		elif header=='Rendering Editors':
			if text=='Render View':
				pm.mel.RenderViewWindow()
			if text=='Render Settings':
				pm.mel.RenderGlobalsWindow()
			if text=='Hypershade':
				pm.mel.HypershadeWindow()
			if text=='Render Setup':
				pm.mel.RenderSetupWindow()
			if text=='Light Editor':
				pm.mel.OpenLightEditor()
			if text=='Custom Stereo Rig Editor':
				pm.mel.OpenStereoRigManager()
			if text=='Rendering Flags':
				pm.mel.RenderFlagsWindow()
			if text=='Shading Group Attributes':
				pm.mel.ShadingGroupAttributeEditor()

		elif header=='Relationship Editors':
			if text=='Animation Layers':
				pm.mel.AnimLayerRelationshipEditor()
			if text=='Camera Sets':
				pm.mel.CameraSetEditor()
			if text=='Character Sets':
				pm.mel.CharacterSetEditor()
			if text=='Deformer Sets':
				pm.mel.DeformerSetEditor()
			if text=='Display Layers':
				pm.mel.LayerRelationshipEditor()
			if text=='Dynamic Relationships':
				pm.mel.DynamicRelationshipEditor()
			if text=='Light Linking: Light Centric':
				pm.mel.LightCentricLightLinkingEditor()
			if text=='Light Linking: Object Centric':
				pm.mel.ObjectCentricLightLinkingEditor()
			if text=='Partitions':
				pm.mel.PartitionEditor()
			if text=='Render Pass Sets':
				pm.mel.RenderPassSetEditor()
			if text=='Sets':
				pm.mel.SetEditor()
			if text=='UV Linking: Texture-Centric':
				pm.mel.TextureCentricUVLinkingEditor()
			if text=='UV Linking: UV-Centric':
				pm.mel.UVCentricUVLinkingEditor()
			if text=='UV Linking: Paint Effects/UV':
				pm.mel.PFXUVSetLinkingEditor()
			if text=='UV Linking: Hair/UV':
				pm.mel.HairUVSetLinkingEditor()


	def cmb000(self, index=-1):
		'''Editors
		'''
		cmb = self.editors_ui.draggable_header.ctxMenu.cmb000

		if index>0:
			text = cmb.items[index]
			if text=='':
				pass
			cmb.setCurrentIndex(0)


	def getEditorWidget(self, name):
		'''Get a maya widget from a given name.

		Parameters:
			name (str): name of widget
		'''
		_name = '_'+name
		if not hasattr(self, _name):
			w = self.convertToWidget(name)
			self.stackedWidget.addWidget(w)
			setattr(self, _name, w)

		return getattr(self, _name)


	def showEditor(self, name, width=640, height=480):
		'''Show, resize, and center the given editor.

		Parameters:
			name (str): The name of the editor.
			width (int): The editor's desired width.
			height (int): The editor's desired height.

		Return:
			(obj) The editor as a QWidget.
		'''
		w = self.getEditorWidget(name)

		self.sb.parent().setUi('dynLayout')
		self.stackedWidget.setCurrentWidget(w)
		self.sb.parent().resize(width, height)
		return w


	def v000(self):
		'''Attributes
		'''
		# e = pm.mel.eval('$tmp=$gAttributeEditorForm')
		# self.showEditor(e, 640, 480)
		pm.mel.AttributeEditor()


	def v001(self):
		'''Outliner
		'''
		# e = pm.mel.eval('$tmp=$gOutlinerForm')

		# if not hasattr(self, 'outlinerEditor_'):
		# 	panel = pm.outlinerPanel()
		# 	self.outliner_ = pm.outlinerPanel(panel, query=True, outlinerEditor=True)
		# 	pm.outlinerEditor(self.outliner_, edit=True, mainListConnection='worldList', selectionConnection='modelList', showShapes=False, showReferenceNodes=False, showReferenceMembers=False, showAttributes=False, showConnected=False, showAnimCurvesOnly=False, autoExpand=False, showDagOnly=True, ignoreDagHierarchy=False, expandConnections=False, showNamespace=True, showCompounds=True, showNumericAttrsOnly=False, highlightActive=True, autoSelectNewObjects=False, doNotSelectNewObjects=False, transmitFilters=False, showSetMembers=True, setFilter='defaultSetFilter', ignoreHiddenAttribute=False)

		# e = pm.outlinerEditor(self.outliner_, edit=True, showSelected=True) #expand to the current selection in the outliner.
		# w = self.showEditor(e, 260, 740)

		# panels = pm.getPanel(type='outlinerPanel')
		# for panel in panels:
		# 	pm.outlinerEditor(panel, edit=1, showSelected=1)
		pm.mel.OutlinerWindow()


	def v002(self):
		'''Tool
		'''
		# e = pm.mel.eval('$tmp=$gToolSettingsForm')
		# self.showEditor(e, 461, 480)
		pm.toolPropertyWindow()


	def v003(self):
		'''Layers
		'''
		# e = pm.mel.eval('$tmp=$gLayerEditorForm')
		# self.showEditor(e, 320, 480)
		# pm.mel.OpenLayerEditor()
		pm.mel.OpenChannelsLayers()


	def v004(self):
		'''Channels
		'''
		# e = pm.mel.eval('$tmp=$gChannelsForm')
		# self.showEditor(e, 320, 640)
		# pm.mel.OpenChannelBox()
		pm.mel.OpenChannelsLayers()


	def v005(self):
		'''Node Editor
		'''
		pm.mel.NodeEditorWindow()


	def v006(self):
		'''Dependancy Graph

		$editorName = ($panelName+"HyperGraphEd");
		hyperGraph -e 
			-graphLayoutStyle "hierarchicalLayout" 
			-orientation "horiz" 
			-mergeConnections 0
			-zoom 1
			-animateTransition 0
			-showRelationships 1
			-showShapes 0
			-showDeformers 0
			-showExpressions 0
			-showConstraints 0
			-showConnectionFromSelected 0
			-showConnectionToSelected 0
			-showConstraintLabels 0
			-showUnderworld 0
			-showInvisible 0
			-transitionFrames 1
			-opaqueContainers 0
			-freeform 0
			-imagePosition 0 0 
			-imageScale 1
			-imageEnabled 0
			-graphType "DAG" 
			-heatMapDisplay 0
			-updateSelection 1
			-updateNodeAdded 1
			-useDrawOverrideColor 0
			-limitGraphTraversal -1
			-range 0 0 
			-iconSize "smallIcons" 
			-showCachedConnections 0
			$editorName // 
		'''
		#e = pm.mel.eval('$tmp=$gHyperGraphPanel')
		# self.showEditor(e, 640, 480)
		pm.mel.HypergraphHierarchyWindow()









#module name
print (__name__)
# --------------------------------------------------------------------------------------------
# Notes
# --------------------------------------------------------------------------------------------

# deprecated: -----------------------------------



# !/usr/bin/python
# coding=utf-8
from uitk.slots.blender import *
from uitk.slots.rigging import Rigging



class Rigging_blender(Rigging, Slots_blender):
	def __init__(self, *args, **kwargs):
		Slots_blender.__init__(self, *args, **kwargs)
		Rigging.__init__(self, *args, **kwargs)

		cmb = self.sb.rigging.draggable_header.ctxMenu.cmb000
		items = []
		cmb.addItems_(items, 'Rigging Editors')

		cmb = self.sb.rigging.cmb001
		items = []
		cmb.addItems_(items, "Create")


	def cmb000(self, index=-1):
		'''Editors
		'''
		cmb = self.sb.rigging.draggable_header.ctxMenu.cmb000

		if index>0:
			text = cmb.items[index]
			if text=='Quick Rig':
				mel.eval('QuickRigEditor;') #Quick Rig
			elif text=='HumanIK':
				mel.eval('HIKCharacterControlsTool;') #HumanIK
			elif text=='Expression Editor':
				mel.eval('ExpressionEditor;') #Expression Editor
			elif text=='Shape Editor':
				mel.eval('ShapeEditor;') #Shape Editor
			elif text=='Connection Editor':
				mel.eval('ConnectionEditor;') #Connection Editor
			elif text=='Channel Control Editor':
				mel.eval('ChannelControlEditor;') #Channel Control Editor
			elif text=='Set Driven Key':
				mel.eval('SetDrivenKeyOptions;') #Set Driven Key
			cmb.setCurrentIndex(0)


	def cmb001(self, index=-1):
		'''Create
		'''
		cmb = self.sb.rigging.cmb001

		if index>0:
			text = cmb.items[index]
			if text=='Joints':
				pm.setToolTo('jointContext') #create joint tool
			elif text=='Locator':
				pm.spaceLocator(p=[0,0,0]) #locator
			elif text=='IK Handle':
				pm.setToolTo('ikHandleContext') #create ik handle
			elif text=='Lattice':
				pm.lattice(divisions=[2,5,2], objectCentered=1, ldv=[2,2,2]) ##create lattice
			elif text=='Cluster':
				mel.eval('CreateCluster;') #create cluster
			cmb.setCurrentIndex(0)


	def chk000(self, state=None):
		'''Scale Joint
		'''
		self.sb.toggleWidgets(setUnChecked='chk001-2')
		self.sb.rigging.tb000.ctxMenu.s000.setValue(pm.jointDisplayScale(query=1)) #init global joint display size


	def chk001(self, state=None):
		'''Scale IK
		'''
		self.sb.toggleWidgets(setUnChecked='chk000, chk002')
		self.sb.rigging.tb000.ctxMenu.setValue(pm.ikHandleDisplayScale(query=1)) #init IK handle display size
		

	def chk002(self, state=None):
		'''Scale IK/FK
		'''
		self.sb.toggleWidgets(setUnChecked='chk000-1')
		self.sb.rigging.tb000.ctxMenu.setValue(pm.jointDisplayScale(query=1, ikfk=1)) #init IKFK display size


	def s000(self, value=None):
		'''Scale Joint/IK/FK
		'''
		value = self.sb.rigging.tb000.ctxMenu.value()

		if self.sb.rigging.chk000.isChecked():
			pm.jointDisplayScale(value) #set global joint display size
		elif self.sb.rigging.chk001.isChecked():
			pm.ikHandleDisplayScale(value) #set global IK handle display size
		else: #self.sb.rigging.chk002.isChecked():
			pm.jointDisplayScale(value, ikfk=1) #set global IKFK display size


	def tb000(self, state=None):
		'''Toggle Display Local Rotation Axes
		'''
		tb = self.sb.rigging.tb000

		joints = pm.ls(type="joint") #get all scene joints

		state = pm.toggle(joints[0], query=1, localAxis=1)
		if tb.ctxMenu.isChecked():
			if not state:
				toggle=True
		else:
			if state:
				toggle=True

		if toggle:
			pm.toggle(joints, localAxis=1) #set display off

		self.mtk.viewportMessage("Display Local Rotation Axes:<hl>"+str(state)+"</hl>")


	def tb001(self, state=None):
		'''Orient Joints
		'''
		tb = self.sb.rigging.tb001

		orientJoint = 'xyz' #orient joints
		if tb.ctxMenu.isChecked():
			orientJoint = 'none' #orient joint to world

		pm.joint(edit=1, orientJoint=orientJoint, zeroScaleOrient=1, ch=1)


	def tb002(self, state=None):
		'''Constraint: Parent
		'''
		tb = self.sb.rigging.tb002

		template = tb.ctxMenu.chk004.isChecked()

		objects = pm.ls(sl=1, objectsOnly=1)

		for obj in objects[:-1]:
			pm.parentConstraint(obj, objects[:-1], maintainOffset=1, weight=1)

			if template:
				if not pm.toggle(obj, template=1, query=1):
					pm.toggle(obj, template=1, query=1)


	@Slots_blender.undoChunk
	def tb003(self, state=None):
		'''Create Locator at Selection
		'''
		tb = self.sb.rigging.tb003

		suffix = tb.ctxMenu.t000.text()
		stripDigits = tb.ctxMenu.chk005.isChecked()
		strip = tb.ctxMenu.t001.text()
		parent = tb.ctxMenu.chk006.isChecked()
		scale = tb.ctxMenu.s001.value()
		lockTranslate = tb.ctxMenu.chk007.isChecked()
		lockRotation = tb.ctxMenu.chk008.isChecked()
		lockScale = tb.ctxMenu.chk009.isChecked()
		freezeTransforms = tb.ctxMenu.chk010.isChecked()
		bakeChildPivot = tb.ctxMenu.chk011.isChecked()
		remove = tb.ctxMenu.chk015.isChecked()

		selection = pm.ls(selection=True)
		self.createLocatorAtObject(selection, suffix=suffix, stripDigits=stripDigits, strip=strip, scale=scale, parent=parent, bakeChildPivot=bakeChildPivot, 
			freezeTransforms=freezeTransforms, lockTranslate=lockTranslate, lockRotation=lockRotation, lockScale=lockScale, remove=remove)


	def tb004(self, state=None):
		'''Lock/Unlock Attributes
		'''
		tb = self.sb.rigging.tb004

		lockTranslate = tb.ctxMenu.chk012.isChecked()
		lockRotation = tb.ctxMenu.chk013.isChecked()
		lockScale = tb.ctxMenu.chk014.isChecked()

		sel = pm.ls(selection=True, transforms=1, long=True)
		for obj in sel:

			attrs_and_state = {
				('tx','ty','tz'):lockTranslate, #attributes and state. ex. ('tx','ty','tz'):False
				('rx','ry','rz'):lockRotation, 
				('sx','sy','sz'):lockScale
			}

			[[pm.setAttr('{}.{}'.format(obj, i), lock=v) for i in k] for k, v in attrs_and_state.items()]


	@Slots.hideMain
	def b000(self):
		'''Object Transform Limit Attributes
		'''
		node = pm.ls(sl=1, objectsOnly=1)
		if not node:
			return 'Error: Operation requires a single selected object.'

		params = ['enableTranslationX','translationX','enableTranslationY','translationY','enableTranslationZ','translationZ',
			'enableRotationX','rotationX','enableRotationY','rotationY','enableRotationZ','rotationZ',
			'enableScaleX','scaleX','enableScaleY','scaleY','enableScaleZ','scaleZ']

		attrs = Slots_blender.getParameterValuesMEL(node, 'transformLimits', params)
		self.setAttributeWindow(node, fn=Slots_blender.setParameterValuesMEL, 'transformLimits', **attrs)


	def b001(self):
		'''Connect Joints
		'''
		pm.connectJoint(cm=1)


	def b002(self):
		'''Insert Joint Tool
		'''
		pm.setToolTo('insertJointContext') #insert joint tool


	def b004(self):
		'''Reroot
		'''
		pm.reroot() #re-root joints


	def b006(self):
		'''Constraint: Point
		'''
		pm.pointConstraint(offset=[0,0,0], weight=1)


	def b007(self):
		'''Constraint: Scale
		'''
		pm.scaleConstraint(offset=[1,1,1], weight=1)


	def b008(self):
		'''Constraint: Orient
		'''
		pm.orientConstraint(offset=[0,0,0], weight=1)


	def b009(self):
		'''Constraint: Aim
		'''
		pm.aimConstraint(offset=[0,0,0], weight=1, aimVector=[1,0,0], upVector=[0,1,0], worldUpType="vector", worldUpVector=[0,1,0])


	def b010(self):
		'''Constraint: Pole Vector
		'''
		pm.orientConstraint(offset=[0,0,0], weight=1)


	@Slots_blender.undoChunk
	def createLocatorAtObject(self, objects, suffix='_LOC', stripDigits=False, strip='', scale=1, parent=False, freezeTransforms=True, 
					bakeChildPivot=True, lockTranslate=False, lockRotation=False, lockScale=False, remove=False, _fullPath=False):
		'''Create locators with the same transforms as any selected object(s).
		If there are vertices selected it will create a locator at the center of the selected vertices bounding box.

		Parameters:
			objects (str)(list): A list of objects, or an object name to create locators at.
			suffix (str): A string appended to the end of the created locators name. (default: '_LOC') '_LOC#'
			stripDigits (bool): Strip numeric characters from the string. If the resulting name is not unique, maya will append a trailing digit. (default=False)
			strip (str): Strip a specific character set from the locator name. The locators name is based off of the selected objects name. (default=None)
			scale (float) = The scale of the locator. (default=1)
			parent (bool): Parent to object to the locator. (default=False)
			freezeTransforms (bool): Freeze transforms on the locator. (default=True)
			bakeChildPivot (bool): Bake pivot positions on the child object. (default=True)
			lockTranslate (bool): Lock the translate values of the child object. (default=False)
			lockRotation (bool): Lock the rotation values of the child object. (default=False)
			lockScale (bool): Lock the scale values of the child object. (default=False)
			remove (bool): Removes the locator and any child locks. (not valid with component selections) (default=False)
			_fullPath (bool): Internal use only (recursion). Use full path names for Dag objects. This can prevent naming conflicts when creating the locator. (default=False)

		Example: createLocatorAtSelection(strip='_GEO', suffix='', stripDigits=True, parent=True, lockTranslate=True, lockRotation=True)
		'''
		def _formatName(name, stripDigits=stripDigits, strip=strip, suffix=suffix):
			if stripDigits:
				name = ''.join([i for i in name if not i.isdigit()])	
			return name.replace(strip, '')+suffix

		def _create_locator(obj, objName, stripDigits=stripDigits, strip=strip, suffix=suffix, scale=scale, _fullPath=_fullPath):
			locName = _formatName(objName, stripDigits, strip, suffix)

			loc = pm.spaceLocator(name=locName)
			if not any([loc, _fullPath]): #if locator creation fails; try again using the objects fullpath name.
				_retry_using_fullPath()

			pm.scale(loc, scale, scale, scale) #scale the locator
			return loc

		def _parent(obj, loc, freezeTransforms=freezeTransforms, bakeChildPivot=bakeChildPivot, state=1):
			objParent = pm.listRelatives(obj, parent=1)
			pm.parent(obj, loc)
			pm.parent(loc, objParent)

			if freezeTransforms:
				_set_lock_state(obj, translate=1, rotate=1, scale=1, state=0) #assure attributes are unlocked.
				pm.makeIdentity(loc, apply=True, normal=1) #1=the normals on polygonal objects will be frozen. 2=the normals on polygonal objects will be frozen only if its a non-rigid transformation matrix.

			if bakeChildPivot:
				pm.select(obj); pm.mel.BakeCustomPivot() #bake pivot on child object. Requires a select

		def _set_lock_state(obj, translate=lockTranslate, rotate=lockRotation, scale=lockScale, state=1):
			if isinstance(obj, str):
				obj = pm.ls(obj.split('|')[-1], transforms=1)[0]

			if type_(obj)=='locator':
				obj = pm.listRelatives(obj, children=1, type='transform')[0]

			if translate: #lock translation values
				[pm.setAttr(getattr(obj, attr), lock=state) for attr in ('tx','ty','tz')] #pm.setAttr(obj[0].translate, lock=state)
			if rotate: #lock rotation values
				[pm.setAttr(getattr(obj, attr), lock=state) for attr in ('rx','ry','rz')]
			if scale: #lock scale values
				[pm.setAttr(getattr(obj, attr), lock=state) for attr in ('sx','sy','sz')]

		def _try_parent_and_lock(obj, loc, objName, parent=parent, _fullPath=_fullPath):
			try:
				if parent: #parent
					_parent(obj, loc)
				_set_lock_state(obj)
			except Exception as error:
				try:
					if not any([loc, _fullPath]): #try again using the objects full path name.
						_retry_using_fullPath()
				except:
					print ('# Error: {}: {} #'.format(objName, error))
					pm.delete(loc)

		def _remove_locators(obj):
			if not type_(obj) == 'locator':
				obj = pm.listRelatives(obj, parent=1)
				if not obj or not type_(obj[0]) == 'locator':
					return False
			pm.ungroup(obj)
			pm.delete(obj)

		type_ = lambda obj: pm.listRelatives(obj, shapes=1)[0].type()

		_retry_using_fullPath = lambda: self.createLocatorAtObject(suffix=suffix, stripDigits=stripDigits, strip=strip, 
			parent=parent, scale=scale, lockTranslate=lockTranslate, lockRotation=lockRotation, lockScale=lockScale, _fullPath=1)

		# pm.undoInfo(openChunk=1)
		objects = pm.ls(objects, long=_fullPath, objectsOnly=True)

		sel_verts = pm.filterExpand(objects, sm=31) #returns a string list.
		if sel_verts: #vertex selection

			objName = sel_verts[0].split('.')[0]
			obj = pm.ls(objName)
			
			loc = _create_locator(obj, objName)

			xmin, ymin, zmin, xmax, ymax, zmax = pm.exactWorldBoundingBox(sel_verts)
			x, y, z = pos = ((xmin + xmax) / 2, (ymin + ymax) / 2, (zmin + zmax) / 2)
			pm.move(x, y, z, loc)

			_try_parent_and_lock(obj, loc, objName)

		else: #object selection
			for obj in objects:

				if remove:
					if pm.objExists(obj): #if the locator hasnt already been deleted by another child.
						_set_lock_state(obj, translate=1, rotate=1, scale=1, state=0)
						_remove_locators(obj)
					continue

				objName = obj.name()

				loc = _create_locator(obj, objName)

				tempConst = pm.parentConstraint(obj, loc, mo=False)
				pm.delete(tempConst)

				_try_parent_and_lock(obj, loc, objName)
		# pm.undoInfo(closeChunk=1)









#module name
print (__name__)
# --------------------------------------------------------------------------------------------
# Notes
# --------------------------------------------------------------------------------------------


#deprecated:

# def createLocatorAtSelection(suffix='_LOC', stripDigits=False, strip='', scale=1, parent=False, freezeChildTransforms=False, bakeChildPivot=False, lockTranslate=False, lockRotation=False, lockScale=False, _fullPath=False):
# 	'''Create locators with the same transforms as any selected object(s).
# 	If there are vertices selected it will create a locator at the center of the selected vertices bounding box.

# 	Parameters:
# 		suffix (str): A string appended to the end of the created locators name. (default: '_LOC') '_LOC#'
# 		stripDigits (bool): Strip numeric characters from the string. If the resulting name is not unique, maya will append a trailing digit. (default=False)
# 		strip (str): Strip a specific character set from the locator name. The locators name is based off of the selected objects name. (default=None)
# 		scale (float) = The scale of the locator. (default=1)
# 		parent (bool): Parent to object to the locator. (default=False)
# 		freezeChildTransforms (bool): Freeze transforms on the child object. (Valid only with parent flag) (default=False)
# 		bakeChildPivot (bool): Bake pivot positions on the child object. (Valid only with parent flag) (default=False)
# 		lockTranslate (bool): Lock the translate values of the child object. (default=False)
# 		lockRotation (bool): Lock the rotation values of the child object. (default=False)
# 		lockScale (bool): Lock the scale values of the child object. (default=False)
# 		_fullPath (bool): Internal use only (recursion). Use full path names for Dag objects. This can prevent naming conflicts when creating the locator. (default=False)

# 	Example:
# 		createLocatorAtSelection(strip='_GEO', suffix='', stripDigits=True, scale=10, parent=True, lockTranslate=True, lockRotation=True)
# 	'''
# 	import pymel.core as pm
# 	sel = pm.ls(selection=True, long=_fullPath, objectsOnly=True)
# 	sel_verts = pm.filterExpand(sm=31)

# 	if not sel:
# 		error = '# Error: Nothing Selected. #'
# 		print (error)
# 		return error

# 	def _formatName(name, stripDigits=stripDigits, strip=strip, suffix=suffix):
# 		if stripDigits:
# 			name_ = ''.join([i for i in name if not i.isdigit()])	
# 		return name_.replace(strip, '')+suffix

# 	def _parent(obj, loc, parent=parent, freezeChildTransforms=freezeChildTransforms, bakeChildPivot=bakeChildPivot):
# 		if parent: #parent
# 			if freezeChildTransforms:
# 				pm.makeIdentity(obj, apply=True, t=1, r=1, s=1, normal=2) #normal parameter: 1=the normals on polygonal objects will be frozen. 2=the normals on polygonal objects will be frozen only if its a non-rigid transformation matrix.
# 			if bakeChildPivot:
# 				pm.select(obj); pm.mel.BakeCustomPivot() #bake pivot on child object.
# 			objParent = pm.listRelatives(obj, parent=1)
# 			pm.parent(obj, loc)
# 			pm.parent(loc, objParent)

# 	def _lockChildAttributes(obj, lockTranslate=lockTranslate, lockRotation=lockRotation, lockScale=lockScale):
# 		try: #split in case of long name to get the obj attribute.  ex. 'f15e_door_61_bellcrank|Bolt_GEO.tx' to: Bolt_GEO.tx
# 			setAttrs = lambda attrs: [pm.setAttr('{}.{}'.format(obj.split('|')[-1], attr), lock=True) for attr in attrs]
# 		except: #if obj is type object:
# 			setAttrs = lambda attrs: [pm.setAttr('{}.{}'.format(obj, attr), lock=True) for attr in attrs]

# 		if lockTranslate: #lock translation values
# 			setAttrs(('tx','ty','tz'))

# 		if lockRotation: #lock rotation values
# 			setAttrs(('rx','ry','rz'))
				
# 		if lockScale: #lock scale values
# 			setAttrs(('sx','sy','sz'))

# 	_fullPath = lambda: self.createLocatorAtSelection(suffix=suffix, stripDigits=stripDigits, 
# 				strip=strip, parent=parent, scale=scale, _fullPath=True, 
# 				lockTranslate=lockTranslate, lockRotation=lockRotation, lockScale=lockScale)

# 	if sel_verts: #vertex selection

# 		objName = sel_verts[0].split('.')[0]
# 		locName = _formatName(objName, stripDigits, strip, suffix)

# 		loc = pm.spaceLocator(name=locName)
# 		if not any([loc, _fullPath]): #if locator creation fails; try again using the objects full path name.
# 			_fullPath()

# 		pm.scale(scale, scale, scale)

# 		bb = pm.exactWorldBoundingBox(sel_verts)
# 		pos = ((bb[0] + bb[3]) / 2, (bb[1] + bb[4]) / 2, (bb[2] + bb[5]) / 2)
# 		pm.move(pos[0], pos[1], pos[2], loc)

# 		_parent(objName, loc)
# 		_lockChildAttributes(objName)

# 	else: #object selection
# 		for obj in sel:

# 			objName = obj.name()
# 			locName = _formatName(objName, stripDigits, strip, suffix)

# 			loc = pm.spaceLocator(name=locName)
# 			if not any([loc, _fullPath]): #if locator creation fails; try again using the objects fullpath name.
# 				_fullPath()

# 			pm.scale(scale, scale, scale)

# 			tempConst = pm.parentConstraint(obj, loc, mo=False)
# 			pm.delete(tempConst)
# 			pm.select(clear=True)

# 			_parent(obj, loc)
# 			_lockChildAttributes(objName)





# !/usr/bin/python
# coding=utf-8
from PySide2 import QtCore, QtGui, QtWidgets

from uitk.widgets.attributes import Attributes
from uitk.widgets.text import RichText, TextOverlay
from uitk.widgets.menu import MenuInstance
from uitk.widgets.pushButton_optionBox import PushButton_optionBox


class PushButtonDraggable(QtWidgets.QPushButton, MenuInstance, Attributes, RichText, TextOverlay):
	'''Draggable/Checkable pushbutton.
	'''
	__mousePressPos = QtCore.QPoint()

	def __init__(self, parent=None, **kwargs):
		QtWidgets.QPushButton.__init__(self, parent)

		self.setStyleSheet(parent.styleSheet()) if parent else None

		self.setCheckable(True)

		self.setStyleSheet('''
			QPushButton {
				border: 1px solid transparent;
				background-color: rgba(127,127,127,2);
			}

			QPushButton::hover {
				background-color: rgba(127,127,127,2);
			}

			QPushButton:checked::hover {
				background-color: rgba(127,127,127,2);
			}''')

		self.setCursor(QtGui.QCursor(QtCore.Qt.OpenHandCursor))

		#override built-ins
		self.text = self.richText
		self.setText = self.setRichText
		self.sizeHint = self.richTextSizeHint

		self.optionBox = None

		self.setAttributes(**kwargs)


	def mousePressEvent(self, event):
		'''
		Parameters:
			event=<QEvent>
		'''
		if event.button()==QtCore.Qt.LeftButton:
			self.__mousePressPos = event.globalPos() #mouse positon at press.
			self.__mouseMovePos = event.globalPos() #mouse move position from last press. (updated on move event) 

			self.setChecked(True) #setChecked to prevent window from closing.
			self.window().preventHide = True

		if event.button()==QtCore.Qt.RightButton:
			self.ctxMenu.show()

		QtWidgets.QPushButton.mousePressEvent(self, event)


	def mouseMoveEvent(self, event):
		'''
		Parameters:
			event=<QEvent>
		'''
		self.setCursor(QtGui.QCursor(QtCore.Qt.ClosedHandCursor))

		#move window:
		curPos = self.window().mapToGlobal(self.window().pos())
		globalPos = event.globalPos()

		try: #if hasattr(self, '__mouseMovePos'):
			diff = globalPos - self.__mouseMovePos

			self.window().move(self.window().mapFromGlobal(curPos + diff))
			self.__mouseMovePos = globalPos
		except AttributeError as error:
			pass

		QtWidgets.QPushButton.mouseMoveEvent(self, event)


	def mouseReleaseEvent(self, event):
		'''
		Parameters:
			event=<QEvent>
		'''
		self.setCursor(QtGui.QCursor(QtCore.Qt.OpenHandCursor))

		moveAmount = event.globalPos() -self.__mousePressPos

		if moveAmount.manhattanLength() >5: #if widget moved:
			self.setChecked(True) #setChecked to prevent window from closing.
			self.window().preventHide = True
		else:
			self.setChecked(not self.isChecked()) #toggle check state

		self.window().preventHide = self.isChecked()
		if not self.window().preventHide: #prevent the parent window from hiding if checked.
			self.window().hide()

		QtWidgets.QPushButton.mouseReleaseEvent(self, event)


	def createOptionBox(self):
		'''
		'''
		self.optionBox = PushButton_optionBox(self) #create an option box
		self.optionBox.create()


	def showEvent(self, event):
		'''
		Parameters:
			event = <QEvent>
		'''
		if self.ctxMenu.containsMenuItems:
			# self.ctxMenu.setTitle(self.text())
			# self.setTextOverlay('⧉', alignment='AlignRight')
			if not self.optionBox:
				self.createOptionBox()

		QtWidgets.QPushButton.showEvent(self, event)


	def hideEvent(self, event):
		'''
		Parameters:
			event = <QEvent>
		'''

		QtWidgets.QPushButton.hideEvent(self, event)









if __name__ == "__main__":
	import sys
	app = QtWidgets.QApplication.instance() or QtWidgets.QApplication(sys.argv) #return the existing QApplication object, or create a new one if none exists.
		
	w = PushButtonDraggable()
	w.show()

	sys.exit(app.exec_())



# -----------------------------------------------------------------------------
# Notes
# -----------------------------------------------------------------------------

'''
Promoting a widget in designer to use a custom class:
>	In Qt Designer, select all the widgets you want to replace, 
		then right-click them and select 'Promote to...'. 

>	In the dialog:
		Base Class:		Class from which you inherit. ie. QWidget
		Promoted Class:	Name of the class. ie. "MyWidget"
		Header File:	Path of the file (changing the extension .py to .h)  ie. myfolder.mymodule.mywidget.h

>	Then click "Add", "Promote", 
		and you will see the class change from "QWidget" to "MyWidget" in the Object Inspector pane.
'''


# Deprecated: --------------------




# !/usr/bin/python
# coding=utf-8
from uitk.slots.max import *
from uitk.slots.crease import Crease



class Crease_max(Crease, Slots_max):
	def __init__(self, *args, **kwargs):
		super().__init__(*args, **kwargs)

		self.creaseValue = 10

		cmb = self.sb.crease.draggable_header.ctxMenu.cmb000
		items = ['Crease Modifier']
		cmb.addItems_(items, 'Crease Modifiers:')


	def cmb000(self, index=-1):
		'''Editors
		'''
		cmb = self.sb.crease.draggable_header.ctxMenu.cmb000

		if index>0:
			text = cmb.items[index]
			if text=='Crease Modifier':
				#check if modifier exists
				for obj in rt.selection:
					mod = obj.modifiers[rt.Crease]
					if mod==None: #if not create
						mod = rt.crease()
						rt.addModifier (obj, mod)
						#set modifier attributes
						# mod.enabled = state

			rt.redrawViews()
			cmb.setCurrentIndex(0)


	def tb000(self, state=None):
		'''Crease
		'''
		tb = self.sb.crease.tb000

		creaseAmount = int(tb.ctxMenu.s003.value())
		normalAngle = int(tb.ctxMenu.s004.value())

		# if tb.ctxMenu.chk011.isChecked(): #crease: Auto
		# 	angleLow = int(tb.ctxMenu.s005.value()) 
		# 	angleHigh = int(tb.ctxMenu.s006.value()) 

		# 	mel.eval("PolySelectConvert 2;") #convert selection to edges
		# 	contraint = pm.polySelectConstraint( mode=3, type=0x8000, angle=True, anglebound=(angleLow, angleHigh) ) # to get edges with angle between two degrees. mode=3 (All and Next) type=0x8000 (edge). 

		creaseAmount = creaseAmount*0.1 #convert to max 0-1 range

		for obj in rt.selection:
			if rt.classOf(obj)=='Editable_Poly':

				if tb.ctxMenu.chk011.isChecked(): #crease: Auto
					minAngle = int(tb.ctxMenu.s005.value()) 
					maxAngle = int(tb.ctxMenu.s006.value()) 

					edgelist = self.getEdgesByAngle(minAngle, maxAngle)
					rt.polyOp.setEdgeSelection(obj, edgelist)

				if tb.ctxMenu.chk004.isChecked(): #crease vertex point
					obj.EditablePoly.setVertexData(1, creaseAmount)
				else: #crease edge
					obj.EditablePoly.setEdgeData(1, creaseAmount)

				if tb.ctxMenu.chk005.isChecked(): #adjust normal angle
					edges = rt.polyop.getEdgeSelection(obj)
					for edge in edges:
						edgeVerts = rt.polyop.getEdgeVerts(obj, edge)
						normal = rt.averageSelVertNormal(obj)
						for vertex in edgeVerts:
							rt.setNormal(obj, vertex, normal)
			else:
				print('Error: object type '+rt.classOf(obj)+' is not supported.')


	def b002(self):
		'''Transfer Crease Edges
		'''
		# an updated version of this is in the maya python projects folder
		# the use of separate buttons for donor and target mesh are obsolete
		# add pm.polySoftEdge (angle=0, constructionHistory=0); #harden edge, when applying crease 

		creaseSet = str(self.sb.crease.b000.text())
		newObject = str(self.sb.crease.b001.text())

		sets = pm.sets (creaseSet, query=1)

		setArray = []
		for set_ in sets:
			name = str(set_)
			setArray.append(name)
		print(setArray)

		pm.undoInfo (openChunk=1)
		for set_ in setArray:
			oldObject = ''.join(set_.partition('.')[:1]) #ex. pSphereShape1 from pSphereShape1.e[260:299]
			pm.select (set_, replace=1)
			value = pm.polyCrease (query=1, value=1)[0]
			name = set_.replace(oldObject, newObject)
			pm.select (name, replace=1)
			pm.polyCrease (value=value, vertexValue=value, createHistory=True)
			# print("crease:", name)
		pm.undoInfo (closeChunk=1)

		self.sb.toggleWidgets(setDisabled='b052', setUnChecked='b000')#,self.sb.crease.b001])
		self.sb.crease.b000.setText("Crease Set")
		# self.sb.crease.b001.setText("Object")









#module name
print (__name__)
# --------------------------------------------------------------------------------------------
# Notes
# --------------------------------------------------------------------------------------------


# !/usr/bin/python
# coding=utf-8
from uitk.slots.max import *
from uitk.slots.scripting import Scripting



class Scripting_max(Scripting, Slots_max):
	def __init__(self, *args, **kwargs):
		super().__init__(*args, **kwargs)

		cmb = self.sb.scripting.draggable_header.ctxMenu.cmb000
		items = ['']
		contents = cmb.addItems_(items, '')


	def cmb000(self, index=-1):
		'''Editors
		'''
		cmb = self.sb.scripting.draggable_header.ctxMenu.cmb000

		if index>0:
			if index==cmd.items.index(''):
				pass
			cmb.setCurrentIndex(0)


	def b000(self):
		'''Toggle Script Output Window
		'''
		state = pm.workspaceControl ("scriptEditorOutputWorkspace", query=1, visible=1)
		pm.workspaceControl ("scriptEditorOutputWorkspace", edit=1, visible=not state)


	def b001(self):
		'''Command Line Window
		'''
		maxEval('commandLineWindow_;')


	def b002(self):
		'''Script Editor
		'''
		maxEval('ScriptEditor;')


	def b003(self):
		'''New Tab
		'''
		label = "Maxscript"
		if self.sb.scripting.chk000.isChecked():
			label = ".py"
		# self.sb.scripting.tabWidget.addTab(label)
		self.sb.scripting.tabWidget.insertTab(0, label)


	def b004(self):
		'''Delete Tab
		'''
		index = self.sb.scripting.tabWidget.currentIndex()
		self.sb.scripting.tabWidget.removeTab(index)









#module name
print (__name__)
# --------------------------------------------------------------------------------------------
# Notes
# --------------------------------------------------------------------------------------------





# !/usr/bin/python
# coding=utf-8
from uitk.slots.maya import *
from uitk.slots.convert import Convert


class Convert_maya(Convert, Slots_maya):
	def __init__(self, *args, **kwargs):
		super().__init__(*args, **kwargs)

		cmb = self.sb.convert.draggable_header.ctxMenu.cmb000
		items = ['']
		cmb.addItems_(items, '')

		cmb = self.sb.convert.cmb001
		items = ['NURBS to Polygons', 'NURBS to Subdiv', 'Polygons to Subdiv', 'Smooth Mesh Preview to Polygons', 'Polygon Edges to Curve', 'Type to Curves', 'Subdiv to Polygons', 'Subdiv to NURBS', 'NURBS Curve to Bezier', 'Bezier Curve to NURBS', 'Paint Effects to NURBS', 'Paint Effects to Curves', 'Texture to Geometry', 'Displacement to Polygons', 'Displacement to Polygons with History', 'Fluid to Polygons', 'nParticle to Polygons', 'Instance to Object', 'Geometry to Bounding Box', 'Convert XGen Primitives to Polygons'] 
		contents = cmb.addItems_(items, 'Convert To')


	def cmb000(self, index=-1):
		'''Editors
		'''
		cmb = self.sb.convert.draggable_header.ctxMenu.cmb000

		if index>0:
			text = cmb.items[index]
			if text=='':
				pass
			cmb.setCurrentIndex(0)


	def cmb001(self, index=-1):
		'''Convert To
		'''
		cmb = self.sb.convert.cmb001

		if index>0:
			text = cmb.items[index]
			if text=='NURBS to Polygons': #index 1
				pm.mel.eval('performnurbsToPoly 0;')
			elif text=='NURBS to Subdiv': #index 2
				pm.mel.eval('performSubdivCreate 0;')
			elif text=='Polygons to Subdiv': #index 3
				pm.mel.eval('performSubdivCreate 0;')
			elif text=='Smooth Mesh Preview to Polygons': #index 4
				pm.mel.eval('performSmoothMeshPreviewToPolygon;')
			elif text=='Polygon Edges to Curve': #index 5
				pm.mel.eval('polyToCurve -form 2 -degree 3 -conformToSmoothMeshPreview 1;')
			elif text=='Type to Curves': #index 6
				pm.mel.eval('convertTypeCapsToCurves;')
			elif text=='Subdiv to Polygons': #index 7
				pm.mel.eval('performSubdivTessellate  false;')
			elif text=='Subdiv to NURBS': #index 8
				pm.mel.eval('performSubdToNurbs 0;')
			elif text=='NURBS Curve to Bezier': #index 9
				pm.mel.eval('nurbsCurveToBezier;')
			elif text=='Bezier Curve to NURBS': #index 10
				pm.mel.eval('bezierCurveToNurbs;')
			elif text=='Paint Effects to NURBS': #index 11
				pm.mel.eval('performPaintEffectsToNurbs  false;')
			elif text=='Paint Effects to Curves': #index 12
				pm.mel.eval('performPaintEffectsToCurve  false;')
			elif text=='Texture to Geometry': #index 13
				pm.mel.eval('performTextureToGeom 0;')
			elif text=='Displacement to Polygons': #index 14
				pm.mel.eval('displacementToPoly;')
			elif text=='Displacement to Polygons with History': #index 15
				pm.mel.eval('setupAnimatedDisplacement;')
			elif text=='Fluid to Polygons': #index 16
				pm.mel.eval('fluidToPoly;')
			elif text=='nParticle to Polygons': #index 17
				pm.mel.eval('particleToPoly;')
			elif text=='Instance to Object': #index 18
				pm.mel.eval('convertInstanceToObject;')
			elif text=='Geometry to Bounding Box': #index 19
				pm.mel.eval('performGeomToBBox 0;')
			elif text=='Convert XGen Primitives to Polygons': #index 20
				import xgenm.xmaya.xgmConvertPrimToPolygon as cpp
				cpp.convertPrimToPolygon(False)

			cmb.setCurrentIndex(0)









#module name
print (__name__)
# --------------------------------------------------------------------------------------------
# Notes
# --------------------------------------------------------------------------------------------


# !/usr/bin/python
# coding=utf-8
from uitk.slots.max import *
from uitk.slots.selection import Selection



class Selection_max(Selection, Slots_max):
	def __init__(self, *args, **kwargs):
		super().__init__(*args, **kwargs)

		cmb = self.sb.selection.draggable_header.ctxMenu.cmb000
		items = ['Selection Set Editor']
		cmb.addItems_(items, 'Selection Editors:')

		cmb = self.sb.selection.cmb002
		items = ['Geometry', 'Shapes', 'Lights', 'Cameras', 'Helpers', 'Space Warps', 'Particle Systems', 'Bone Objects']
		cmb.addItems_(items, 'Select by Type:')

		cmb = self.sb.selection.cmb003
		items = ['Vertex', 'Edge', 'Border', 'Face', 'Element']
		cmb.addItems_(items, 'Convert To:')

		cmb = self.sb.selection.cmb005
		items = ['Angle', 'Border', 'Edge Loop', 'Edge Ring', 'Shell', 'UV Edge Loop']
		cmb.addItems_(items, 'Off')


	@property
	def currentSelection(self):
		'''Gets the currently selected objects or object components.

		Return:
			(array) current selection as a maxscript array.
		'''
		sel = rt.selection
		if not sel:
			return 'Error: Nothing Selected.'

		level = rt.subObjectLevel
		if level in (0, None): #objs
			s = [i for i in sel]
		elif level==1: #verts
			s = Slots_max.getComponents(sel[0], 'vertices', selection=True)
		elif level==2: #edges
			s = Slots_max.getComponents(sel[0], 'edges', selection=True)
		elif level==3: #borders
			s = rt.getBorderSelection(sel[0])
		elif level==4: #faces
			s = Slots_max.getComponents(sel[0], 'faces', selection=True)

		return rt.array(*s) #unpack list s and convert to an array.


	def txt001(self):
		'''Select By Name
		'''
		searchStr = str(self.sb.selection.txt001.text()) #asterisk denotes startswith*, *endswith, *contains* 
		if searchStr:
			selection = rt.select(searchStr)


	def lbl000(self):
		'''Selection Sets: Create New
		'''
		cmb = self.sb.selection.cmb001
		if not cmb.isEditable():
			cmb.addItems_('', ascending=True)
			cmb.setEditable(True)
			cmb.lineEdit().setPlaceholderText('New Set:')
		else:
			name = cmb.currentText()
			self.creatNewSelectionSet(name)
			self.cmb001() #refresh the sets comboBox
			cmb.setCurrentIndex(0)


	def lbl001(self):
		'''Selection Sets: Modify Current
		'''
		cmb = self.sb.selection.cmb001
		if not cmb.isEditable():
			name = cmb.currentText()
			self._oldSetName = name
			cmb.setEditable(True)
			cmb.lineEdit().setPlaceholderText(name)
		else:
			name = cmb.currentText()
			self.modifySet(self._oldSetName)
			cmb.setItemText(cmb.currentIndex(), name)
			# self.cmb001() #refresh the sets comboBox


	def lbl002(self):
		'''Selection Sets: Delete Current
		'''
		cmb = self.sb.selection.cmb001
		name = cmb.currentText()

		set_ = self.getSet(name)
		set_array = self.getSet(name, 'set_array')

		if set_:
			rt.deleteItem(set_array, set_)

		self.cmb001() #refresh the sets comboBox


	def lbl003(self):
		'''Grow Selection
		'''
		# expand functionalitly to grow according to selection type
		#grow line #PolytoolsSelect.Pattern7 1
		#grow loop #PolytoolsSelect.GrowLoop()
		#grow ring #PolytoolsSelect.GrowRing()
		for obj in rt.selection:
			obj.EditablePoly.GrowSelection()


	def lbl004(self):
		'''Shrink Selection
		'''
		for obj in rt.selection:
			obj.EditablePoly.ShrinkSelection()


	def lbl005(self):
		'''Selection Sets: Select Current
		'''
		cmb = self.sb.selection.cmb001
		name = cmb.currentText()

		set_ = self.getSet(name)
		obj = self.getSet(name, 'object')
		level = self.getSet(name, 'objectLevel')

		if obj:
			rt.select(obj)
			rt.subObjectLevel = level #set the appropriate object level.
		if set_:
			rt.select(set_)

		rt.redrawViews()


	def chk004(self, state=None):
		'''Ignore Backfacing (Camera Based Selection)
		'''
		for obj in rt.selection:
			if self.selection_submenu_ui.chk004.isChecked():
				sel.ignoreBackfacing = True
				self.sb.messageBox('Camera-based selection <hl>On</hl>.', messageType='Result')
			else:
				sel.ignoreBackfacing = False
				self.sb.messageBox('Camera-based selection <hl>Off</hl>.', messageType='Result')


	def chk008(self, state=None):
		'''Toggle Soft Selection
		'''
		if self.selection_submenu_ui.chk008.isChecked():
			pm.softSelect(edit=1, softSelectEnabled=True)
			self.sb.messageBox('Soft Select <hl>On</hl>.', messageType='Result')
		else:
			pm.softSelect(edit=1, softSelectEnabled=False)
			self.sb.messageBox('Soft Select <hl>Off</hl>.', messageType='Result')


	def cmb000(self, index=-1):
		'''Editors
		'''
		cmb = self.sb.selection.draggable_header.ctxMenu.cmb000

		if index>0:
			text = cmb.items[index]
			if text=='Selection Set Editor':
				maxEval('macros.run "Edit" "namedSelSets"')
			cmb.setCurrentIndex(0)


	def cmb001(self, index=-1):
		'''Selection Sets
		'''
		cmb = self.sb.selection.cmb001

		sets_ = self.getSelectionSets(rt.geometry)
		cmb.addItems_([s for s in sets_], clear=True)


	def cmb002(self, index=-1):
		'''Select All Of Type
		'''
		cmb = self.sb.selection.cmb002

		if index>0:
			text = cmb.items[index]
			if text=='Geometry': #Select all Geometry
				rt.select(rt.geometry)
			elif text=='Shapes': #Select all Geometry
				rt.select(rt.shapes)
			elif text=='Lights': #Select all Geometry
				rt.select(rt.lights)
			elif text=='Cameras': #Select all Geometry
				rt.select(rt.cameras)
			elif text=='Helpers': #Select all Geometry
				rt.select(rt.helpers)
			elif text=='Space Warps': #Select all Geometry
				rt.select(rt.spacewarps)
			elif text=='Particle Systems': #Select all Geometry
				rt.select(rt.particelsystems)
			elif text=='Bone Objects': #Select all Geometry
				rt.select(rt.boneobjects)

			cmb.setCurrentIndex(0)


	def cmb003(self, index=-1):
		'''Convert To
		'''
		cmb = self.sb.selection.cmb003

		if index>0:
			text = cmb.items[index]
			for obj in rt.selection:
				for i in cmb.items:
					if index==cmb.items.index(i):
						obj.convertSelection('CurrentLevel', i) #Convert current selection to index of string i
						# rt.setSelectionLevel(obj, i) #Change component mode to i
						rt.subObjectLevel = cmb.items.index(i) #the needed component level corresponds to the item's index.
			cmb.setCurrentIndex(0)


	def cmb005(self, index=-1):
		'''Selection Contraints
		'''
		cmb = self.sb.selection.cmb005

		if index>0:
			text = cmb.items[index]
			if text=='Angle':
				mel.eval('dR_selConstraintAngle;') #dR_DoCmd("selConstraintAngle");
			elif text=='Border':
				mel.eval('dR_selConstraintBorder;') #dR_DoCmd("selConstraintBorder");
			elif text=='Edge Loop':
				mel.eval('dR_selConstraintEdgeLoop;') #dR_DoCmd("selConstraintEdgeLoop");
			elif text=='Edge Ring':
				mel.eval('dR_selConstraintEdgeRing;') #dR_DoCmd("selConstraintEdgeRing");
			elif text=='Shell':
				mel.eval('dR_selConstraintElement;') #dR_DoCmd("selConstraintElement");
			elif text=='UV Edge Loop':
				mel.eval('dR_selConstraintUVEdgeLoop;') #dR_DoCmd("selConstraintUVEdgeLoop");
		else:
			mel.eval('dR_selConstraintOff;') #dR_DoCmd("selConstraintOff");


	def cmb006(self, index=-1):
		'''Currently Selected Objects
		'''
		cmb = self.sb.selection.draggable_header.ctxMenu.cmb006

		cmb.clear()
		items = [str(i) for i in rt.selection]
		widgets = [cmb.menu_.add('QCheckBox', setText=t, setChecked=1) for t in items[:50]] #selection list is capped with a slice at 50 elements.

		for w in widgets:
			try:
				w.disconnect() #disconnect all previous connections.
			except TypeError:
				pass #if no connections are present; pass
			w.toggled.connect(lambda state, widget=w: self.chkxxx(state=state, widget=widget))


	def chkxxx(self, **kwargs):
		'''Transform Constraints: Constraint CheckBoxes
		'''
		try:
			if kwargs['state']==True:
				rt.deselect(kwargs['widget'].text())
			else:
				rt.select(kwargs['widget'].text())
		except KeyError:
			pass


	def tb000(self, state=None):
		'''Select Nth
		'''
		tb = self.sb.selection.tb000

		edgeRing = tb.ctxMenu.chk000.isChecked()
		edgeLoop = tb.ctxMenu.chk001.isChecked()
		pathAlongLoop = tb.ctxMenu.chk009.isChecked()
		shortestPath = tb.ctxMenu.chk002.isChecked()
		borderEdges = tb.ctxMenu.chk010.isChecked()
		step = tb.ctxMenu.s003.value()


		if edgeRing: # rt.macros.run('PolyTools', 'Ring')
			obj = rt.selection[0]
			self.selectRing(obj)

		elif edgeLoop: #rt.macros.run('PolyTools', 'Loop')
			obj = rt.selection[0]
			self.selectLoop(obj)

		elif pathAlongLoop:
			pm.select(self.getPathAlongLoop(selection))

		elif shortestPath:
			pm.select(self.getShortestPath(selection))

		elif borderEdges:
			pm.select(self.getBorderComponents(selection, returnCompType='edges'))


	def tb001(self, state=None):
		'''Select Similar
		'''
		tb = self.sb.selection.tb001

		tolerance = str(tb.ctxMenu.s000.value()) #string value because mel.eval is sending a command string
		
		level = rt.subObjectLevel
		if level is 0: #object
			maxEval('actionMan.executeAction 0 "40099"') #Selection: Select Similar
		else:
			print ('# Error: No support for sub-object level selections. #')


	def tb002(self, state=None):
		'''Select Island: Select Polygon Face Island
		'''
		tb = self.sb.selection.tb002

		rangeX = float(tb.ctxMenu.s002.value())
		rangeY = float(tb.ctxMenu.s004.value())
		rangeZ = float(tb.ctxMenu.s005.value())

		curmod = rt.Modpanel.getcurrentObject()
		curmod.selectAngle = rangeX
		curmod.selectByAngle = not curmod.selectByAngle

		# $.selectAngle=rangeX
		# $.selectByAngle = on
		# sel = $.selectedfaces as bitarray #maintains current single selection. need to reselect with angle contraint active to make work.
		# $.selectByAngle = off
		# print(sel)
		# setFaceSelection sel #{}


	def tb003(self, state=None):
		'''Select Edges By Angle
		'''
		tb = self.sb.selection.tb003

		angleLow = tb.ctxMenu.s006.value()
		angleHigh = tb.ctxMenu.s007.value()

		objects = pm.ls(sl=1, objectsOnly=1)
		edges = Slots_max.getEdgesByNormalAngle(objects, lowAngle=angleLow, highAngle=angleHigh)
		rt.select(edges)


	def setSelectionStyle(self, ctx):
		'''Set the selection style context.

		Parameters:
			ctx (str): Selection style context. valid: 'marquee', 'circular', 'fence', 'lasso', 'paint'.
		'''
		if ctx=='marquee':
			maxEval('actionMan.executeAction 0 "59232"') #Rectangular select region
		elif ctx=='circular':
			maxEval('actionMan.executeAction 0 "59233"') #Circular select region
		elif ctx=='fence':
			maxEval('actionMan.executeAction 0 "59234"') #Fence select region
		elif ctx=='lasso':
			maxEval('actionMan.executeAction 0 "59235"') #Lasso select region
		elif ctx=='paint':
			maxEval('actionMan.executeAction 0 "59236"') #Paint select region


	def generateUniqueSetName(self):
		'''Generate a generic name based on the object's name.

		<objectName>_Set<int>
		'''
		num = ptk.cycle(list(range(99)), 'selectionSetNum')
		name = '{0}_Set{1}'.format(rt.selection[0].name, num) #ie. pCube1_Set0
		return name


	def creatNewSelectionSet(self, name=None):
		'''Selection Sets: Create a new selection set.

		Parameters:
			name (str): The desired name of the new set.
		'''
		if rt.isValidObj(name): # obj!=rt.undefined
			self.sb.messageBox('Set with name <hl>{}</hl> already exists.'.format(name))
			return

		else: #create set
			sel = self.currentSelection
			if sel:
				if not name:
					name = self.generateUniqueSetName()

				try: #object level set.
					rt.selectionSets[name] = sel #create a standard object set.
				except IndexError: #sub-object level set.
					set_array = self.getSetArray(rt.selection[0], rt.subObjectLevel) #ie. rt.selection[0].faces
					set_array[name] = sel #create a sub-object level set for the selected currently selected components.
			else:
				self.sb.messageBox('Nothing selected.')
				return


	def modifySet(self, name):
		'''Selection Sets: Modify Current by renaming or changing the set members.

		Parameters:
			name (str): Name of an existing selection set.
		'''
		sel = self.currentSelection
		if sel:
			newName = self.sb.selection.cmb001.currentText()
			if not newName:
				newName = self.generateUniqueSetName()

			set_ = self.getSet(name)

			try: #object level set.
				set_.name = newName #rename the old set with the comboBoxes current text.
				rt.selectionSets[name] = sel #create a standard object set.
			except IndexError: #sub-object level set.
				set_array = self.getSet(name, 'set_array')
				set_array[name] = sel #create a sub-object level set for the selected currently selected components.
				# if not newName==name:
				# 	rt.deleteItem(set_array, set_) #delete the old if a new set name is given.
		else:
			self.sb.messageBox('Nothing selected.')
			return


	def getSet(self, name, index=0, objects=[]):
		'''Get a set or set info by name.

		Parameters:
			name (str): Set name.
			index (str)(int): Desired return value type. Valid values are: 0:'set'(default), 1:'object', 2:'objectLevel' 4:'set_array'.
			objects (list): The group of objects to get the set from.

		Return:
			depending on the given index:
			(obj) <set> <object>
			(int) <object level>
			(array) <set_array>

		ex. self.getSet(name, 'set_array') #using string 'set_array' as index value instead of int 4 for readability.
		'''
		if not objects:
			objects = rt.geometry
		sets = self.getSelectionSets(objects)

		try:
			if index in ('set', 0):
				value = sets[name][0]
			elif index in ('object', 1):
				value = sets[name][1]
			elif index in ('objectLevel', 2):
				value = sets[name][2]
			elif index in ('set_array', 3):
				value = sets[name][3]
		except (IndexError, KeyError):
			value = None

		return value


	def getSelectionSets(self, objects=[], includeEmptySets=True):
		'''Get selection sets for a group of objects in the given list.
		Returns Object and Sub-Object Level sets.

		Parameters:
			objects (list): The objects to get sets from. ie. rt.cameras (default is rt.geometry)

		Return:
			(dict) {'set name':[<set>, <object>, <object level as int>, <set array>]}
		'''
		if includeEmptySets:
			objects = [i for i in objects]+[None]

		sets={}
		for level in [0,1,2,4]: #range(5) omitting 'borders' because of attribute error: <obj.borders>
			for obj in objects:
				_sets = self.selectionSets(obj, level)
				for setName, set_ in _sets.items():
					set_array = self.getSetArray(obj, level)
					sets[str(setName)] = [set_, obj, level, set_array]

		return sets


	def getSetArray(self, obj=None, index=0):
		'''Get the array containing a set by array type.

		Parameters:
			obj (obj): Parent obj of the array.
			index (int): Array type. 

		Return:
			(array) maxscript array object.
		'''
		type_ = {0:'', 1:'vertices', 2:'edges', 3:'borders', 4:'faces'}

		if index==0:
			_set_array = rt.selectionSets
		else:
			_set_array = getattr(obj, type_[index], None) #ie. <obj.faces>

		return _set_array


	def selectionSets(self, obj=None, level=None):
		'''Gets any existing selection sets for the given object.

		Parameters:
			obj (obj): The object to get sets for. If no object is given, any empty sets will be returned.
			level (int): The sub-object level. Valid values are: 0(obj), 1(vertices), 2(edges), 3(borders), 4(faces)

		Return:
			(dict) {'set name':<set>}
		'''
		if level is None: #if level isn't given:
			level = rt.subObjectLevel #use the current object level.

		set_array = self.getSetArray(obj, level)

		try:
			if obj is None: #empty sets
				sets = {s.name:s for s in rt.selectionSets if len(s)==0}
			elif level in (0, None):
				sets = {s.name:s for n,s in enumerate(rt.selectionSets, 1) for i in range(1, rt.getNamedSelSetItemCount(n)+1) if rt.getNamedSelSetItem(n, i)==obj}
			elif level==1: #verts
				sets = {s:set_array[s] for s in obj.vertices.selSetNames}
			elif level==2: #edges
				sets = {s:set_array[s] for s in obj.edges.selSetNames}
			elif level==3: #borders
				sets = {s:set_array[s] for s in obj.borders.selSetNames}
			elif level==4: #faces
				sets = {s:set_array[s] for s in obj.faces.selSetNames}
		except AttributeError: #skip if the obj type doesn't have an attribute of the given level: ie. object has no attribute 'selSetNames' (ex. non-editable mesh)
			sets = {}

		return sets


	def selectLoop(self, obj):
		'''Select a component loop from two or more selected adjacent components (or a single edge).

		Parameters:
			obj (obj): An Editable polygon object.

		ex. obj = rt.selection[0]
			selectLoop(obj)
		'''
		level = rt.subObjectLevel
		if level is 1: #vertex
			obj.convertselection('Vertex',  'Edge', requireAll=True)
			obj.SelectEdgeLoop()
			obj.convertselection('Edge', 'Vertex')

		elif level is 2: #edge
			obj.SelectEdgeLoop()

		elif level is 4: #face
			obj.convertselection('Face', 'Edge', requireAll=True)
			obj.SelectEdgeRing()
			obj.convertselection('Edge', 'Face')

		rt.redrawViews()


	def selectRing(self, obj):
		'''Select a component ring from two or more selected adjacent components (or a single edge).

		Parameters:
			obj (obj): An Editable polygon object.

		ex. obj = rt.selection[0]
			selectRing(obj)
		'''
		level = rt.subObjectLevel
		if level is 1: #vertex
			obj.convertselection('Vertex',  'Edge', requireAll=True)
			obj.SelectEdgeRing()
			obj.convertselection('Edge', 'Vertex')

		elif level is 2: #edge
			obj.SelectEdgeRing()

		elif level is 4: #face
			obj.convertselection('Face', 'Edge', requireAll=True)
			obj.SelectEdgeLoop()
			obj.convertselection('Edge', 'Face')

		rt.redrawViews()









#module name
print (__name__)
# --------------------------------------------------------------------------------------------
# Notes
# --------------------------------------------------------------------------------------------


# !/usr/bin/python
# coding=utf-8
from uitk.slots.max import *
from uitk.slots.file import File



class File_max(File, Slots_max):
	def __init__(self, *args, **kwargs):
		super().__init__(*args, **kwargs)

		cmb = self.sb.file.draggable_header.ctxMenu.cmb000
		items = ['Schematic View']
		cmb.addItems_(items, 'File Editors')

		cmb = self.sb.file.cmb002
		ctx = cmb.ctxMenu
		if not ctx.containsMenuItems:
			ctx.add('QPushButton', setObjectName='b000', setText='Open Directory', setToolTip='Open the autosave directory.') #open directory
			ctx.add('QPushButton', setObjectName='b002', setText='Delete All', setToolTip='Delete all autosave files.') #delete all
			ctx.add('QCheckBox', setText='Autosave', setObjectName='chk006', setChecked=rt.autosave.Enable, setToolTip='Set the autosave state as active or disabled.') #toggle autosave
			ctx.add('QSpinBox', setPrefix='Amount: ', setObjectName='s000', setMinMax_='1-100 step1', setValue=rt.autosave.NumberOfFiles, setHeight_=20, setToolTip='The number of autosave files to retain.') #autosave amount
			ctx.add('QSpinBox', setPrefix='Interval: ', setObjectName='s001', setMinMax_='1-60 step1', setValue=rt.autosave.Interval, setHeight_=20, setToolTip='The autosave interval in minutes.') #autosave interval
			ctx.chk006.toggled.connect(lambda s: rt.autosave.setmxsprop('Enable', s))
			ctx.s000.valueChanged.connect(lambda v: rt.autosave.setmxsprop('NumberOfFiles', v))
			ctx.s001.valueChanged.connect(lambda v: rt.autosave.setmxsprop('Interval', v))
			cmb.addItems_(self.getRecentAutosave(appendDatetime=True), 'Recent Autosave', clear=True)

		cmb = self.sb.file.cmb003
		cmb.addItems_(['Import file', 'Import Options', 'Merge', 'Replace', 'Link Revit', 'Link FBX', 'Link AutoCAD'], 'Import')

		cmb = self.sb.file.cmb004
		items = ["Export Selection", "Export Options", "Unreal", "Unity", "GoZ", "Send to Maya: New Scene", "Send to Maya: Update Scene", "Send to Maya: Add to Scene"]
		cmb.addItems_(items, "Export")


	def cmb000(self, index=-1):
		'''Editors
		'''
		cmb = self.sb.file.draggable_header.ctxMenu.cmb000

		if index>0:
			text = cmb.items[index]
			if text=='Schematic View':
				maxEval('schematicView.Open "Schematic View 1"')
			cmb.setCurrentIndex(0)


	def cmb001(self, index=-1):
		'''Recent Projects
		'''
		cmb = self.sb.file.cmb006.ctxMenu.cmb001

		items = cmb.addItems_(self.getRecentProjects(), "Recent Projects", clear=True)

		if index>0:
			maxEval('setProject "'+items[index]+'"')
			cmb.setCurrentIndex(0)


	def cmb002(self, index=-1):
		'''Recent Autosave
		'''
		cmb = self.sb.file.cmb002

		items = cmb.addItems_(self.getRecentAutosave(appendDatetime=True), "Recent Autosave", clear=True)

		if index>0:
			file = Slots.timeStamp(cmb.items[index], detach=True)[0] #cmb.items[index].split('\\')[-1]
			rt.loadMaxFile(file)
			cmb.setCurrentIndex(0)


	def cmb003(self, index=-1):
		'''Import
		'''
		cmb = self.sb.file.cmb003

		if index>0: #hide then perform operation
			self.sb.parent().hide(force=1)
			if index == 1: #Import
				maxEval('max file import')
			elif index == 2: #Import options
				maxEval('')
			elif index == 3: #Merge
				maxEval('max file merge')
			elif index == 4: #Replace
				maxEval('max file replace')
			elif index == 5: #Manage Links: Link Revit File
				maxEval('actionMan.executeAction 769996349 "108"')
			elif index == 6: #Manage Links: Link FBX File
				maxEval('actionMan.executeAction 769996349 "109"')
			elif index == 7: #Manage Links: Link AutoCAD File
				maxEval('actionMan.executeAction 769996349 "110"')
			cmb.setCurrentIndex(0)


	def cmb004(self, index=-1):
		'''Export
		'''
		cmb = self.sb.file.cmb004

		if index>0: #hide then perform operation
			self.sb.parent().hide(force=1)
			if index==1: #Export selection
				maxEval('actionMan.executeAction 0 "40373"') #max file export
			elif index==2: #Export options
				maxEval('')
			elif index==3: #Unreal: File: Game Exporter
				maxEval('actionMan.executeAction 0 "40488"')
			elif index==4: #Unity: File: Game Exporter
				maxEval('actionMan.executeAction 0 "40488"')
			elif index==5: #GoZ
				print('GoZ')
				maxEval(''' 
					try (
						if (s_verbose) then print "\n === 3DS -> ZBrush === "
						local result = s_gozServer.GoToZBrush()
						) catch ();
					''')
			elif index==6: #One Click Maya: Send as New Scene to Maya
				maxEval('actionMan.executeAction 924213374 "0"')
			elif index==7: #One Click Maya: Update Current Scene in Maya
				maxEval('actionMan.executeAction 924213374 "1"')
			elif index==8: #One Click Maya: Add to Current Scene in Maya
				maxEval('actionMan.executeAction 924213374 "2"')

			cmb.setCurrentIndex(0)


	def cmb005(self, index=-1):
		'''Recent Files
		'''
		cmb = self.sb.file.cmb005

		items = cmb.addItems_(self.getRecentFiles(), 'Recent Files:', clear=True)

		if index>0:
			rt.loadMaxFile(str(items[index-1]))
			cmb.setCurrentIndex(0)


	def cmb006(self, index=-1):
		'''Project Folder
		'''
		cmb = self.sb.file.cmb006

		path = self.formatPath(rt.pathconfig.getCurrentProjectFolderPath(), strip='file') #current project path.
		items = [f for f in os.listdir(path)]

		project = self.formatPath(path, 'dir') #add current project path string to label. strip path and trailing '/'

		cmb.addItems_(items, project, clear=True)

		if index>0:
			dir_ = path+items[index-1] #reformat for network address
			os.startfile(dir_)
			cmb.setCurrentIndex(0)


	def tb000(self, state=None):
		'''Save
		'''
		tb = self.sb.file.draggable_header.ctxMenu.tb000

		wireframe = tb.ctxMenu.chk000.isChecked()
		increment = tb.ctxMenu.chk001.isChecked()
		quit = tb.ctxMenu.chk002.isChecked()

		if wireframe:
			pm.mel.DisplayWireframe()

		if increment:
			pm.mel.IncrementAndSave()
		else:
			rt.saveMaxFile(quiet=True)

		if quit: #quit maya
			import time
			for timer in range(5):
				self.mtk.viewportMessage('Shutting Down:<hl>'+str(timer)+'</hl>')
				time.sleep(timer)
			pm.mel.quit() # pm.Quit()


	def lbl000(self):
		'''Set Project
		'''
		maxEval('macros.run "Tools" "SetProjectFolder"') #rt.pathconfig.setCurrentProjectFolderPath(path) --Sets the current project folder to the given path. The Project Folder is displayed in the title bar of 3ds Max and will be updated instantly.

		self.cmb006() #refresh cmb006 items to reflect new project folder


	def lbl001(self):
		'''Minimize Main Application
		'''
		app = rt.createOLEObject('Shell.Application')
		maxEval('minimizeAll app')
		maxEval('undoMinimizeAll app')
		rt.releaseOLEObject(app)
		self.sb.parent().hide(force=1)


	def lbl002(self):
		'''Restore Main Application
		'''
		maxEval('actionMan.executeAction 0 "50026"') #Tools: Maximize Viewport Toggle


	def lbl003(self):
		'''Close Main Application
		'''
		maxEval("quitmax #noprompt") # maxEval("quitmax")


	def lbl004(self):
		'''Open current project root
		'''
		recentFiles = self.getRecentFiles() #current project path.
		dir_ = self.formatPath(recentFiles) #reformat for network address
		os.startfile(dir_)


	def b000(self):
		'''Autosave: Open Directory
		'''
		dir_ = rt.GetDir(rt.name('autoback'))
			
		try:
			os.startfile(self.formatPath(dir_))
		except FileNotFoundError as error:
			self.sb.messageBox('The system cannot find the file specified.')


	def b002(self):
		'''Autosave: Delete All
		'''
		files = self.getRecentAutosave()
		for file in files:
			try:
				os.remove(file)
			except Exception as error:
				print (error)


	def b015(self):
		'''Remove String From Object Names.
		'''
		from_ = str(self.sb.file.t000.text()) #asterisk denotes startswith*, *endswith, *contains* 
		to = str(self.sb.file.t001.text())
		replace = self.sb.file.chk004.isChecked()
		selected = self.sb.file.chk005.isChecked()

		# objects = pm.ls (from_) #Stores a list of all objects starting with 'from_'
		# if selected:
		# 	objects = pm.ls (selection=1) #if use selection option; get user selected objects instead
		# from_ = from_.strip('*') #strip modifier asterisk from user input

		# for obj in objects:
		# 	relatives = pm.listRelatives(obj, parent=1) #Get a list of it's direct parent
		# 	if 'group*' in relatives: #If that parent starts with group, it came in root level and is pasted in a group, so ungroup it
		# 		relatives[0].ungroup()

		# 	newName = to
		# 	if replace:
		# 		newName = obj.replace(from_, to)
		# 	pm.rename(obj, newName) #Rename the object with the new name


	def getRecentFiles(self, index=None):
		'''Get a list of recent files from "RecentDocuments.xml" in the maxData directory.

		Parameters:
			index (int): Return the recent file directory path at the given index. Index 0 would be the most recent file.

		Return:
			(list)(str)
		'''
		maxEval('''
		fn _getRecentFiles = (
			local recentfiles = (getdir #maxData) + "RecentDocuments.xml"
			if doesfileexist recentfiles then (
				XMLArray = #()
				xDoc = dotnetobject "system.xml.xmldocument"
				xDoc.Load recentfiles
				Rootelement = xDoc.documentelement

				XMLArray = for i = 0 to rootelement.childnodes.item[4].childnodes.itemof[0].childnodes.count-1 collect (
					rootelement.childnodes.item[4].childnodes.itemof[0].childnodes.itemof[i].childnodes.itemof[3].innertext
				)

				Return XMLArray
				LRXML = Undefined
				XDoc = Undefined
				XDoc = nothing
			)
		)
		''')
		files = rt._getRecentFiles()
		result = [self.formatPath(f) for f in files]

		try:
			return result[index]
		except (IndexError, TypeError) as error:
			return result


	def getRecentProjects(self):
		'''Get a list of recently set projects.

		Return:
			(list)
		'''
		files = ['No 3ds max function']
		result = [self.formatPath(f) for f in list(reversed(files))]

		return result


	def getRecentAutosave(self, appendDatetime=False):
		'''Get a list of autosave files.

		Parameters:
			appendDatetime (bool): Attach a modified timestamp and date to given file path(s).

		Return:
			(list)
		'''
		from datetime import datetime

		path = rt.GetDir(rt.name('autoback'))
		files = [r'{}\{}'.format(path, f) for f in os.listdir(path) if f.endswith('.max') or f.endswith('.bak')] #get list of max autosave files
		result = [self.formatPath(f) for f in list(reversed(files))] #format and reverse the list.

		if appendDatetime:  #attach modified timestamp
			result = Slots.timeStamp(result)

		return result


	def incrementFileName(self, fileName):
		'''Increment the given file name.

		Parameters:
			fileName (str): file name with extension. ie. elise_mid.ma

		Return:
			(str) incremented name. ie. elise_mid.000.ma
		'''
		import re

		#remove filetype extention
		currentName = fileName[:fileName.rfind('.')] #name without extension ie. elise_mid.009 from elise_mid.009.mb
		#get file number
		numExt = re.search(r'\d+$', currentName) #check if the last chars are numberic
		if numExt is not None:
			name = currentName[:currentName.rfind('.')] #strip off the number ie. elise_mid from elise_mid.009
			num = int(numExt.group())+1 #get file number and add 1 ie. 9 becomes 10
			prefix = '000'[:-len(str(num))]+str(num) #prefix '000' removing zeros according to num length ie. 009 becomes 010
			newName = name+'.'+prefix #ie. elise_mid.010
			
		else:
			newName = currentName+'.001'

		return newName


	def deletePreviousFiles(self, fileName, path, numberOfPreviousFiles=5):
		'''Delete older files.

		Parameters:
			fileName (str): file name with extension. ie. elise_mid.ma
			numberOfPreviousFiles (int): Number of previous copies to keep.
		'''
		import re, os

		#remove filetype extention
		currentName = fileName[:fileName.rfind('.')] #name without extension ie. elise_mid.009 from elise_mid.009.mb
		#get file number
		numExt = re.search(r'\d+$', currentName) #check if the last chars are numberic
		if numExt is not None:
			name = currentName[:currentName.rfind('.')] #strip off the number ie. elise_mid from elise_mid.009
			num = int(numExt.group())+1 #get file number and add 1 ie. 9 becomes 10

			oldNum = num-numberOfPreviousFiles
			oldPrefix = '000'[:-len(str(oldNum))]+str(oldNum) #prefix the appropriate amount of zeros in front of the old number
			oldName = name+'.'+oldPrefix #ie. elise_mid.007
			try: #search recursively through the project folder and delete any old folders with the old filename
				dir_ =  os.path.abspath(os.path.join(path, "../.."))
				for root, directories, files in os.walk(dir_):
					for filename in files:
						if all([filename==oldName+ext for ext in ('.ma','.ma.swatches','.mb','.mb.swatches')]):
							try:
								import os
								os.remove(filename)
							except:
								pass
			except OSError:
				print('{0} {1}'.format('Error: Could not delete', path+oldName))









#module name
print (__name__)
# --------------------------------------------------------------------------------------------
# Notes
# --------------------------------------------------------------------------------------------


# !/usr/bin/python
# coding=utf-8
from uitk.slots import Slots


class Nurbs(Slots):
	'''
	'''
	def __init__(self, *args, **kwargs):
		super().__init__(*args, **kwargs)
		'''
		'''
		dh = self.sb.nurbs.draggable_header
		dh.ctxMenu.add(self.sb.ComboBox, setObjectName='cmb000', setToolTip='Maya Curve Operations')

		tb000 = self.sb.nurbs.tb000
		tb000.ctxMenu.add('QSpinBox', setPrefix='Degree:', setObjectName='s002', setValue=3, setMinMax_='0-9999 step1', setToolTip='The degree of the resulting surface.')
		tb000.ctxMenu.add('QSpinBox', setPrefix='Start Sweep:', setObjectName='s003', setValue=3, setMinMax_='0-360 step1', setToolTip='	The value for the start sweep angle.')
		tb000.ctxMenu.add('QSpinBox', setPrefix='End Sweep:', setObjectName='s004', setValue=3, setMinMax_='0-360 step1', setToolTip='The value for the end sweep angle.')
		tb000.ctxMenu.add('QSpinBox', setPrefix='Sections:', setObjectName='s005', setValue=8, setMinMax_='0-9999 step1', setToolTip='The number of surface spans between consecutive curves in the loft.')
		tb000.ctxMenu.add('QCheckBox', setText='Range', setObjectName='chk006', setChecked=False, setToolTip='Force a curve range on complete input curve.')
		tb000.ctxMenu.add('QCheckBox', setText='Polygon', setObjectName='chk007', setChecked=True, setToolTip='The object created by this operation.')
		tb000.ctxMenu.add('QCheckBox', setText='Auto Correct Normal', setObjectName='chk008', setChecked=False, setToolTip='Attempt to reverse the direction of the axis in case it is necessary to do so for the surface normals to end up pointing to the outside of the object.')
		tb000.ctxMenu.add('QCheckBox', setText='Use Tolerance', setObjectName='chk009', setChecked=False, setToolTip='Use the tolerance, or the number of sections to control the sections.')
		tb000.ctxMenu.add('QDoubleSpinBox', setPrefix='Tolerance:', setObjectName='s006', setValue=0.001, setMinMax_='0-9999 step.001', setToolTip='Tolerance to build to (if useTolerance attribute is set).')

		tb001 = self.sb.nurbs.tb001
		tb001.ctxMenu.add('QCheckBox', setText='Uniform', setObjectName='chk000', setChecked=True, setToolTip='The resulting surface will have uniform parameterization in the loft direction. If set to false, the parameterization will be chord length.')
		tb001.ctxMenu.add('QCheckBox', setText='Close', setObjectName='chk001', setChecked=False, setToolTip='The resulting surface will be closed (periodic) with the start (end) at the first curve. If set to false, the surface will remain open.')
		tb001.ctxMenu.add('QSpinBox', setPrefix='Degree:', setObjectName='s000', setValue=3, setMinMax_='0-9999 step1', setToolTip='The degree of the resulting surface.')
		tb001.ctxMenu.add('QCheckBox', setText='Auto Reverse', setObjectName='chk002', setChecked=False, setToolTip='The direction of the curves for the loft is computed automatically. If set to false, the values of the multi-use reverse flag are used instead.')
		tb001.ctxMenu.add('QSpinBox', setPrefix='Section Spans:', setObjectName='s001', setValue=1, setMinMax_='0-9999 step1', setToolTip='The number of surface spans between consecutive curves in the loft.')
		tb001.ctxMenu.add('QCheckBox', setText='Range', setObjectName='chk003', setChecked=False, setToolTip='Force a curve range on complete input curve.')
		tb001.ctxMenu.add('QCheckBox', setText='Polygon', setObjectName='chk004', setChecked=True, setToolTip='The object created by this operation.')
		tb001.ctxMenu.add('QCheckBox', setText='Reverse Surface Normals', setObjectName='chk005', setChecked=True, setToolTip='The surface normals on the output NURBS surface will be reversed. This is accomplished by swapping the U and V parametric directions.')
		tb001.ctxMenu.add('QCheckBox', setText='Angle Loft Between Two Curves', setObjectName='chk010', setChecked=False, setToolTip='Perform a loft at an angle between two selected curves or polygon edges (that will be extracted as curves).')
		tb001.ctxMenu.add('QSpinBox', setPrefix='Angle Loft: Spans:', setObjectName='s007', setValue=6, setMinMax_='2-9999 step1', setToolTip='Angle loft: Number of duplicated points (spans).')



# !/usr/bin/python
# coding=utf-8
from uitk.slots.maya import *
from uitk.slots.mirror import Mirror



class Mirror_maya(Mirror, Slots_maya):
	def __init__(self, *args, **kwargs):
		super().__init__(*args, **kwargs)

		cmb = self.sb.mirror.draggable_header.ctxMenu.cmb000
		items = ['']
		cmb.addItems_(items, '')


	def cmb000(self, index=-1):
		'''Editors
		'''
		cmb = self.sb.mirror.draggable_header.ctxMenu.cmb000

		if index>0:
			if index==cmd.items.index(''):
				pass
			cmb.setCurrentIndex(0)


	@Slots_maya.attr
	def tb000(self, state=None):
		'''Mirror Geometry
		'''
		tb = self.sb.mirror.tb000

		axis = self.sb.getAxisFromCheckBoxes('chk000-3', tb.ctxMenu)
		axisPivot = 2 if tb.ctxMenu.chk008.isChecked() else 1 #1) object space, 2) world space.
		cutMesh = tb.ctxMenu.chk005.isChecked() #cut mesh on axis before mirror.
		uninstance = tb.ctxMenu.chk009.isChecked() #Un-Instance any previously instanced objects before mirroring.
		instance = tb.ctxMenu.chk004.isChecked()
		merge = tb.ctxMenu.chk007.isChecked()
		mergeMode = tb.ctxMenu.s001.value()
		mergeThreshold = tb.ctxMenu.s000.value()
		deleteOriginal = tb.ctxMenu.chk010.isChecked() #delete the original objects after mirroring.
		deleteHistory = tb.ctxMenu.chk006.isChecked() #delete the object's non-deformer history.

		objects = pm.ls(sl=1)

		self.sb.duplicate.slots.unInstance(objects)

		return self.mirrorGeometry(objects, axis=axis, axisPivot=axisPivot, cutMesh=cutMesh, instance=instance, merge=merge, 
			mergeMode=mergeMode, mergeThreshold=mergeThreshold, deleteOriginal=deleteOriginal, deleteHistory=deleteHistory)


	def b000(self):
		'''Mirror: X
		'''
		self.sb.mirror.tb000.ctxMenu.chk001.setChecked(True)
		self.tb000()


	def b001(self):
		'''Mirror: Y
		'''
		self.sb.mirror.tb000.ctxMenu.chk002.setChecked(True)
		self.tb000()


	def b002(self):
		'''Mirror: Z
		'''
		self.sb.mirror.tb000.ctxMenu.chk003.setChecked(True)
		self.tb000()


	@mtk.undo
	def mirrorGeometry(self, objects=None, axis='-x', axisPivot=2, cutMesh=False, instance=False, 
					merge=False, mergeMode=1, mergeThreshold=0.005, deleteOriginal=False, deleteHistory=True):
		'''Mirror geometry across a given axis.

		Parameters:
			objects (obj): The objects to mirror. If None; any currently selected objects will be used.
			axis (string) = The axis in which to perform the mirror along. case insensitive. (valid: 'x', '-x', 'y', '-y', 'z', '-z')
			axisPivot (int): The pivot on which to mirror on. valid: 0) Bounding Box, 1) Object, 2) World.
			cutMesh (bool): Perform a delete along specified axis before mirror.
			instance (bool): Instance the mirrored object(s).
			merge (bool): Merge the mirrored geometry with the original.
			mergeMode (int): 0) Do not merge border edges. 1) Border edges merged. 2) Border edges extruded and connected.
			mergeThreshold (float) = Merge vertex distance.
			deleteOriginal (bool): Delete the original objects after mirroring.
			deleteHistory (bool): Delete non-deformer history on the object before performing the operation.

		Return:
			(obj) The polyMirrorFace history node if a single object, else None.
		'''
		direction = {
			'-x': (0, 0,(-1, 1, 1)),	# the direction dict:
			 'x': (1, 0,(-1, 1, 1)),	# 	first index: axis direction: 0=negative axis, 1=positive.
			'-y': (0, 1, (1,-1, 1)),	# 	second index: axis_as_int: 0=x, 1=y, 2=z
			 'y': (1, 1, (1,-1, 1)),	# 	remaining three are (x, y, z) scale values. #Used only when scaling an instance.
			'-z': (0, 2, (1, 1,-1)),
			 'z': (1, 2, (1, 1,-1)),
		}

		axis = axis.lower() #assure case.
		axisDirection, axis_as_int, scale = direction[axis] #ex. (1, 5, (1, 1,-1)) broken down as: axisDirection=1, axis_as_int=5, scale: (x=1, y=1, z=-1)

		if not objects:
			objects = pm.ls(sl=1)
			if not objects:
				self.sb.messageBox('<b>Nothing selected.<b><br>Operation requires at least one selected polygon object.')
				return

		# pm.undoInfo(openChunk=1)
		for obj in pm.ls(objects, objectsOnly=1):
			if deleteHistory:
				pm.mel.BakeNonDefHistory(obj)

			if cutMesh:
				self.sb.edit.slots.deleteAlongAxis(obj, axis) #delete mesh faces that fall inside the specified axis.

			if instance: #create instance and scale negatively
				inst = pm.instance(obj) # bt_convertToMirrorInstanceMesh(0); #x=0, y=1, z=2, -x=3, -y=4, -z=5
				pm.xform(inst, scale=scale) #pm.scale(z,x,y, pivot=(0,0,0), relative=1) #swap the xyz values to transform the instanced node
				return inst if len(objects)==1 else inst 

			else: #mirror
				print ('axis:',axis_as_int)
				polyMirrorFaceNode = pm.ls(pm.polyMirrorFace(obj, axis=axis_as_int, axisDirection=axisDirection, mirrorAxis=axisPivot, mergeMode=mergeMode, 
						mirrorPosition=0, mergeThresholdType=1, mergeThreshold=mergeThreshold, smoothingAngle=30, flipUVs=0, ch=1))[0] #mirrorPosition x, y, z - This flag specifies the position of the custom mirror axis plane

				if not merge:
					polySeparateNode = pm.ls(pm.polySeparate(obj, uss=1, inp=1))[2]

					pm.connectAttr(polyMirrorFaceNode.firstNewFace, polySeparateNode.startFace, force=True) 
					pm.connectAttr(polyMirrorFaceNode.lastNewFace, polySeparateNode.endFace, force=True)

		if deleteOriginal and not merge:
			for obj in objects:
				pm.delete(obj)
		try:
			if len(objects)==1:
				return polyMirrorFaceNode
		except AttributeError as error:
			return None
		# pm.undoInfo(closeChunk=1)









#module name
print (__name__)
# --------------------------------------------------------------------------------------------
# Notes
# --------------------------------------------------------------------------------------------



#deprecated:


# if axis=='X': #'x'
# 			axisDirection = 0 #positive axis
# 			axis_ = 0 #axis
# 			x=-1; y=1; z=1 #scale values

# 		elif axis=='-X': #'-x'
# 			axisDirection = 1 #negative axis
# 			axis_ = 1 #0=-x, 1=x, 2=-y, 3=y, 4=-z, 5=z 
# 			x=-1; y=1; z=1 #if instance: used to negatively scale

# 		elif axis=='Y': #'y'
# 			axisDirection = 0
# 			axis_ = 2
# 			x=1; y=-1; z=1

# 		elif axis=='-Y': #'-y'
# 			axisDirection = 1
# 			axis_ = 3
# 			x=1; y=-1; z=1

# 		elif axis=='Z': #'z'
# 			axisDirection = 0
# 			axis_ = 4
# 			x=1; y=1; z=-1

# 		elif axis=='-Z': #'-z'
# 			axisDirection = 1
# 			axis_ = 5
# 			x=1; y=1; z=-1

	# def chk000(self, state=None):
	# 	'''
	# 	Delete: Negative Axis. Set Text Mirror Axis
	# 	'''
	# 	axis = "X"
	# 	if self.sb.mirror.chk002.isChecked():
	# 		axis = "Y"
	# 	if self.sb.mirror.chk003.isChecked():
	# 		axis = "Z"
	# 	if self.sb.mirror.chk000.isChecked():
	# 		axis = '-'+axis
	# 	self.sb.mirror.tb000.setText('Mirror '+axis)
	# 	self.sb.mirror.tb003.setText('Delete '+axis)


	# #set check states
	# def chk000(self, state=None):
	# 	'''
	# 	Delete: X Axis
	# 	'''
	# 	self.sb.toggleWidgets(setUnChecked='chk002,chk003')
	# 	axis = "X"
	# 	if self.sb.mirror.chk000.isChecked():
	# 		axis = '-'+axis
	# 	self.sb.mirror.tb000.setText('Mirror '+axis)
	# 	self.sb.mirror.tb003.setText('Delete '+axis)


	# def chk002(self, state=None):
	# 	'''
	# 	Delete: Y Axis
	# 	'''
	# 	self.sb.toggleWidgets(setUnChecked='chk001,chk003')
	# 	axis = "Y"
	# 	if self.sb.mirror.chk000.isChecked():
	# 		axis = '-'+axis
	# 	self.sb.mirror.tb000.setText('Mirror '+axis)
	# 	self.sb.mirror.tb003.setText('Delete '+axis)


	# def chk003(self, state=None):
	# 	'''
	# 	Delete: Z Axis
	# 	'''
	# 	self.sb.toggleWidgets(setUnChecked='chk001,chk002')
	# 	axis = "Z"
	# 	if self.sb.mirror.chk000.isChecked():
	# 		axis = '-'+axis
	# 	self.sb.mirror.tb000.setText('Mirror '+axis)
	# 	self.sb.mirror.tb003.setText('Delete '+axis)


	# def chk005(self, state=None):
		# '''
		# Mirror: Cut
		# '''
		#keep menu and submenu in sync:
		# if self.mirror_submenu.chk005.isChecked():
		# 	self.sb.toggleWidgets(setChecked='chk005')
		# else:
		# 	self.sb.toggleWidgets(setUnChecked='chk005')


# !/usr/bin/python
# coding=utf-8
import sys

from PySide2 import QtCore

from uitk.tcl import Tcl
from mayatk import getMainWindow


class Tcl_maya(Tcl):
	'''Tcl class overridden for use with Autodesk Maya.

	Parameters:
		parent = Application top level window instance.
	'''
	def __init__(self, parent=None, slots='slots/maya', *args, **kwargs):
		'''
		'''
		if not parent:
			try:
				parent = getMainWindow()

			except Exception as error:
				print(__file__, error)

		super().__init__(parent, slots=slots, *args, **kwargs)


	def keyPressEvent(self, event):
		'''
		Parameters:
			event = <QEvent>
		'''
		if not event.isAutoRepeat():
			modifiers = self.app.keyboardModifiers()

			if event.key()==self.key_undo and modifiers==QtCore.Qt.ControlModifier:
				import Pymel.Core as pm
				pm.undo()

		Tcl.keyPressEvent(self, event)


	def showEvent(self, event):
		'''
		Parameters:
			event = <QEvent>
		'''

		Tcl.showEvent(self, event) #super().showEvent(event)


	def hideEvent(self, event):
		'''
		Parameters:
			event = <QEvent>
		'''
		if __name__ == "__main__":
			self.app.quit()
			sys.exit() #assure that the sys processes are terminated.

		Tcl.hideEvent(self, event) #super().hideEvent(event)

# --------------------------------------------------------------------------------------------

_instances = {}
def getInstance(instanceID=None, *args, **kwargs):
	'''Get an instance of this class using a given instanceID.
	The instanceID is either the object or the object's id.

	Parameters:
		instanceID () = The instanceID can be any immutable type.
		args/kwargs () = The args to be passed to the class instance when it is created.

	Return:
		(obj) An instance of this class.

	Example: tcl = Tcl_maya.getInstance(id(0), key_show='Key_F12') #returns the class instance with an instance ID of the value of `id(0)`.
	'''
	import inspect

	if instanceID is None:
		instanceID = inspect.stack()[1][3]
	try:
		return _instances[instanceID]

	except KeyError as error:
		_instances[instanceID] = Tcl_maya(*args, **kwargs)
		return _instances[instanceID]


def show(instanceID=None, *args, **kwargs):
	'''Expands `getInstance` to get and then show an instance in a single command.
	'''
	inst = getInstance(instanceID=instanceID, *args, **kwargs)
	inst.show()

# --------------------------------------------------------------------------------------------









# --------------------------------------------------------------------------------------------

if __name__ == "__main__":

	main = Tcl_maya()
	main.show('init')

	exit_code = main.app.exec_()
	if exit_code != -1:
		sys.exit(exit_code) # run app, show window, wait for input, then terminate program with a status code returned from app.

#module name
print (__name__)
# --------------------------------------------------------------------------------------------
# Notes
# --------------------------------------------------------------------------------------------

# Example startup macro:

	# def hk_tentacle_show():
	# 	'''Display the uitk marking menu.
	# 	'''
	# 	if 'tcl' not in globals():
	# 		from tcl_maya import Tcl_maya
	# 		global tcl
	# 		tcl = Tcl_maya(key_show='Key_F12', profile=False)

	# 	tcl.sendKeyPressEvent(tcl.key_show)



# deprecated: -----------------------------------


	# _instances = {}

	# def __init__(self, parent=None, id=None, slots='slots/maya', *args, **kwargs):
	# 	'''
	# 	'''
	# 	if not parent:
	# 		try:
	# 			parent = getMainWindow()

	# 		except Exception as error:
	# 			print(__file__, error)

	# 	super().__init__(parent, slots=slots, *args, **kwargs)

	# 	if id is not None:
	# 		Tcl_maya._instances[id] = self


	# @classmethod
	# def get_instance(cls, id, profile=False):
	# 	if id not in cls._instances:
	# 		cls._instances[id] = Tcl_maya(key_show='Key_F12', profile=profile, id=id)
	# 	return cls._instances[id]


	# def show_instance(self):
	# 	self.sendKeyPressEvent(self.key_show)

	# #call
	# from uitk.tcl_maya import Tcl_maya
	# 	tcl = Tcl_maya.get_instance(id, profile=profile)
	# 	tcl.show_instance()



# class Instance(Instance):
# 	'''Manage multiple instances of Tcl_maya.
# 	'''
# 	def __init__(self, *args, **kwargs):
# 		'''
# 		'''
# 		super().__init__(*args, **kwargs)
# 		self.Class = Tcl_maya





# if not pm.runTimeCommand('Hk_main', exists=1):
# 	pm.runTimeCommand(
# 		'Hk_main'
# 		annotation='',
# 		catagory='',
# 		commandLanguage='python',
# 		command=if 'uitk' not in {**locals(), **globals()}: main = Tcl_maya.createInstance(); main.hide(); main.show(),
# 		hotkeyCtx='',
# 	)


	# def hk_main_show():
	# 	'''hk_main_show
	# 	Display main marking menu.

	# 	profile: Prints the total running time, times each function separately, and tells you how many times each function was called.
	# 	'''
	# 	if 'main' not in locals() and 'main' not in globals():
	# 		from main_maya import Instance
	# 		main = Instance()

	# 	main.show_()
	# 	# import cProfile
	# 	# cProfile.run('main.show_()')